import numpy
import random
from MAST.structopt.tools.find_defects import find_defects

def lattice_alteration(indiv, Optimizer):
    """Move function to perform Lattice Alteration of atoms
    Inputs:
        indiv = Individual class object to be altered
        Optimizer = Optimizer class object with needed parameters
    Outputs:
        indiv = Altered Individual class object
    """
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    cell_max=numpy.maximum.reduce(indiv[0].get_positions())
    cell_min=numpy.minimum.reduce(indiv[0].get_positions())
    if Optimizer.structure=='Defect':
        if Optimizer.isolate_mutation:
            indc,indb,vacant,swap,stro = find_defects(indiv[0],Optimizer.solidbulk,0)
            positions=indc.get_positions()
        else:
            positions=indiv[0].get_positions()
    else:
        positions=indiv[0].get_positions()
    if len(positions) != 0:
        try:
            natomsmove=random.randint(1,len(positions)/5)
        except ValueError:
            natomsmove=1
        ratmlocnew=[0]*natomsmove
        for i in range(natomsmove):
            ratmloc=random.randint(0,len(positions)-1)
            ratmlocnew[i]=(random.uniform(cell_min[0],cell_max[0]), random.uniform(cell_min[1],cell_max[1]), random.uniform(cell_min[2],cell_max[2]))
            positions[ratmloc]=ratmlocnew[i]
        if Optimizer.structure=='Defect':
            if Optimizer.isolate_mutation:
                indc.set_positions(positions)
                indiv[0]=indc.copy()
                indiv[0].extend(indb)
            else:
                indiv[0].set_positions(positions)
        else:
            indiv[0].set_positions(positions)
    else:
        natomsmove=0
        ratmlocnew=0
    Optimizer.output.write('Lattice Alteration Mutation performed on individual\n')
    Optimizer.output.write('Index = '+repr(indiv.index)+'\n')
    Optimizer.output.write('Number of atoms moved = '+repr(natomsmove)+'\n')
    Optimizer.output.write(repr(ratmlocnew)+'\n')
    Optimizer.output.write(repr(indiv[0])+'\n')
    muttype='LA'+repr(natomsmove)
    if indiv.energy==0:
        indiv.history_index=indiv.history_index+'m'+muttype
    else:
        indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv


'''General functions for use in optimizer'''
from BestInds import *
from check_atomlist_concentration import *
from eval_energy import eval_energy
#from fitness_switch import *
from get_best import *
from setup_calculator import *
from setup_fixed_region_calculator import *
from find_defects import *
from calc_dist import *
from find_top_layer import *
from rattle import *
from convert_time import *
from remove_duplicates import *
try:
	from StemCalc import ConvStem
except:
	pass


##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
import os
import shutil
import logging
import time
from MAST.utility import MASTObj
from MAST.utility import MASTError
from MAST.utility import dirutil
from MAST.utility import Metadata
from MAST.utility import loggerutils
from pymatgen.core.structure import Structure
from pymatgen.io.vaspio import Poscar
from pymatgen.io.cifio import CifParser

class BaseChecker(MASTObj):
    """Base checker class. This class switches between
        program-specific functions.
        For example, phon-related functions or vasp-related
        functions get their own checker class.
    """
    
    def __init__(self, allowed_keys, **kwargs):
        allowed_keys_base = dict()
        allowed_keys_base.update(allowed_keys) 
        MASTObj.__init__(self, allowed_keys_base, **kwargs)
        self.logger = logging.getLogger(self.keywords['name'])
        self.logger = loggerutils.add_handler_for_recipe(self.keywords['name'], self.logger)

    def is_frozen(self, output_filename=""):
        """Check if the ingredient is frozen.
            Args:
                output_filename <str>: Output filename to check (not full path)
                    e.g. OUTCAR
            Returns:
                True if frozen, False otherwise
        """
        if output_filename == "":
            self.logger.error("No output filename given for is_frozen check")
            return False
        frozensec=21000
        if "mast_frozen_seconds" in self.keywords['program_keys'].keys():
            frozensec = int(self.keywords['program_keys']['mast_frozen_seconds'])
        st = os.stat(os.path.join(self.keywords['name'], output_filename)) 
        if time.time() - st.st_mtime > frozensec: 
            self.logger.warning("Ingredient appears frozen based on file %s" % output_filename)
            return True
        else:
            return False

    def is_complete(self):
        raise NotImplementedError
    def is_started(self):
        raise NotImplementedError
    def is_ready_to_run(self):
        raise NotImplementedError
    def get_coordinates_only_structure_from_input(self):
        """Get coordinates-only structures from mast_coordinates
            ingredient keyword
            Args:
                keywords <dict>: ingredient keywords
            Returns:
                coordstrucs <list>: list of Structure objects
        """
        coordposlist=self.keywords['program_keys']['mast_coordinates']
        coordstrucs=list()
        coordstruc=None
        for coordpositem in coordposlist:
            if ('poscar' in os.path.basename(coordpositem).lower()):
                coordstruc = Poscar.from_file(coordpositem).structure
            elif ('cif' in os.path.basename(coordpositem).lower()):
                coordstruc = CifParser(coordpositem).get_structures()[0]
            else:
                error = 'Cannot build structure from file %s' % coordpositem
                raise MASTError(self.__class__.__name__, error)
            coordstrucs.append(coordstruc)
        return coordstrucs
    def softlink_a_file(self, childpath, filename):
        """Softlink a parent file to a matching name in the child folder.
            Args:
                childpath <str>: path to child ingredient
                filename <str>: file name (e.g. "CHGCAR")
        """
        parentpath = self.keywords['name']
        dirutil.lock_directory(childpath)
        import subprocess
        #print "cwd: ", os.getcwd()
        #print parentpath
        #print childpath
        if os.path.isfile("%s/%s" % (parentpath, filename)):
            if not os.path.isfile("%s/%s" % (childpath, filename)):
                curpath = os.getcwd()
                os.chdir(childpath)
                mylink=subprocess.Popen("ln -s %s/%s %s" % (parentpath,filename,filename), shell=True)
                mylink.wait()
                os.chdir(curpath)
            else:
                self.logger.warning("%s already exists in %s. Parent %s not softlinked." % (filename,childpath,filename))
        else:
            raise MASTError(self.__class__.__name__,"No file in parent path %s named %s. Cannot create softlink." % (parentpath, filename))
        dirutil.unlock_directory(childpath)
    
    def copy_a_file(self, childpath, pfname, cfname):
        """Copy a parent file to an arbitrary name in the child folder.
            Args:
                childpath <str>: path to child ingredient
                pfname <str>: file name in parent folder (e.g. "CONTCAR")
                cfname <str>: file name for child folder (e.g. "POSCAR")
        """
        parentpath = self.keywords['name']
        dirutil.lock_directory(childpath)
        self.logger.info("Attempting copy of %s/%s into %s/%s" % (parentpath, pfname, childpath, cfname))
        if os.path.isfile("%s/%s" % (parentpath, pfname)):
            if not os.path.isfile("%s/%s" % (childpath, cfname)):
                shutil.copy("%s/%s" % (parentpath, pfname),"%s/%s" % (childpath, cfname))
            else:
                self.logger.warning("%s already exists in %s. Parent file %s not copied from %s into child %s." % (cfname,childpath,pfname,parentpath,cfname))
        else:
            raise MASTError(self.__class__.__name__,"No file in parent path %s named %s. Cannot copy into child path as %s." % (parentpath, pfname, cfname))
        dirutil.unlock_directory(childpath)



from MAST.structopt.fingerprinting import get_fingerprint
from MAST.structopt.tools import get_best
from MAST.structopt.tools import remove_duplicates
from MAST.structopt.switches import selection_switch, moves_switch, lambdacommamu
import random

def mutation_dups_quench(pop, Optimizer):
    """Predator function that removes individuals based on fitness and mutates replacements
    Also quenches top individuals
    """
    fitlist = [one.fitness for one in pop]
    nfitlist, nindices = remove_duplicates(fitlist, Optimizer.demin)
    STR = ''
    newpop = []
    if len(nfitlist) != len(fitlist):
        STR+='Predator: Removed total of '+repr(len(fitlist)-len(nfitlist))+' from population\n'
    otherlist = []
    for i in range(len(pop)):
        if i not in nindices:
            STR+='Predator: Removed '+repr(pop[i].history_index)+'\n'
            otherlist.append(pop[i])
        else:
            newpop.append(pop[i])
    while len(newpop) < Optimizer.nindiv:
        indiv = random.choice(otherlist).duplicate()
        indiv, scheme = moves_switch(indiv,Optimizer)
        indiv.energy = 1000
        indiv.fitness = 1000
        newpop.append(indiv)
        STR+='Predator: Adding mutated duplicates to new pop history='+indiv.history_index+'\n'
        nindices.append(indiv.index)
    nindices.sort()
    if Optimizer.natural_selection_scheme=='fussf':
        for ind in newpop:
            if ind.fingerprint == 0:
                ind.fingerprint = get_fingerprint(Optimizer,ind,Optimizer.fpbin,Optimizer.fpcutoff)
    if 'lambda,mu' in Optimizer.algorithm_type:
        try:
            mark = [ index for index,n in enumerate(nindices) if n > Optimizer.nindiv-1][0]
        except:
            mark = Optimizer.nindiv
        Optimizer.mark = mark
        pop, str1 = lambdacommamu.lambdacommamu(newpop, Optimizer)
        STR+=str1
    else:
        pop = selection_switch(newpop, Optimizer.nindiv, Optimizer.natural_selection_scheme, Optimizer)
    pop = get_best(pop,len(pop))
    
    if Optimizer.genrep >10:
        from MAST.structopt.moves.quench import quench
        import os
        olammpsvar = os.environ['LAMMPS_COMMAND']
        try:
            from mpi4py import MPI
            if '-n' in olammpsvar:
                lcommand = olammpsvar.split('-n')
                lcommand[1]=lcommand[1].split()
                nproc = MPI.COMM_WORLD.Get_size()
                os.environ['LAMMPS_COMMAND'] = '{0}-n {1} {2}'.format(lcommand[0],nproc,lcommand[1][1])
        except:
            pass
        oqns2 = Optimizer.quench_n_steps_2
        Optimizer.quench_n_steps_2 = 100000
        opar = Optimizer.parallel
        Optimizer.parallel = False
        for i in range(3):
            pop[i] = quench(pop[i],Optimizer)
        Optimizer.quench_n_steps_2 = oqns2
        os.environ['LAMMPS_COMMAND'] = olammpsvar
        Optimizer.parallel = opar
    return pop, STR



"""Tests for Chopingredient"""

from MAST.ingredients.chopingredient import ChopIngredient

import unittest
from unittest import SkipTest
import os
import time
import MAST
import pymatgen
from MAST.utility import dirutil
from MAST.utility import MASTFile
import shutil
import numpy as np
testname="chop_test_run"
testdir = dirutil.get_test_dir(testname)
old_control = os.getenv("MAST_CONTROL")
old_recipe = os.getenv("MAST_RECIPE_PATH")
old_scratch = os.getenv("MAST_SCRATCH")

class TestRunIngredient(unittest.TestCase):

    def setUp(self):
        os.environ['MAST_CONTROL'] = testdir + "/test_control"
        os.environ['MAST_RECIPE_PATH'] = testdir
        os.environ['MAST_SCRATCH'] = testdir
        os.chdir(testdir)

        if not os.path.isdir("writedir"):
            os.mkdir("writedir")
        if not os.path.isdir("test_control"):
            os.mkdir("test_control")
        if not os.path.isdir("writedir/next_ingred"):
            os.mkdir("writedir/next_ingred")
        if not os.path.isdir("writedir/single_label1"):
            os.mkdir("writedir/single_label1")
        if not os.path.isdir("writedir/neb_labelinit-labelfin_stat"):
            os.mkdir("writedir/neb_labelinit-labelfin_stat")
        if not os.path.isdir("writedir/single_phonon_label1"):
            os.mkdir("writedir/single_phonon_label1")
        if not os.path.isfile("test_control/set_platform"):
            shutil.copy(old_control + "/set_platform", "test_control/set_platform")
        if not os.path.isdir("test_control/programkeys"):
            shutil.copytree(old_control + "/programkeys", "test_control/programkeys")

    def tearDown(self):
        tearlist = list()
        #tearlist.append("writedir/single_label1")
        #tearlist.append("writedir/next_ingred")
        #tearlist.append("writedir/neb_labelinit-labelfin_stat")
        #tearlist.append("writedir/single_phonon_label1")
        tearlist.append("writedir")
        tearlist.append("test_control")
        for tearfolder in tearlist:
            shutil.rmtree(tearfolder)
        os.environ['MAST_CONTROL'] = old_control
        os.environ['MAST_RECIPE_PATH'] = old_recipe
        os.environ['MAST_SCRATCH'] = old_scratch

    def test___init__(self):
        self.assertTrue(True)
        #raise SkipTest
        #self.testclass.__init__(**kwargs)

    def test_run_singlerun(self):
        #raise SkipTest
        ingdir="%s/writedir/single_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        kdict['mast_kpoints'] = [2,2,2,"M"]
        kdict['mast_xc'] = 'pw91'
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        mywr = ChopIngredient(name=ingdir, program_keys = kdict, structure=my_structure)
        mywr.write_singlerun()
        myri = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_structure)
        myri.run_singlerun()
        self.assertTrue(myri.checker.is_ready_to_run())
        mysubmit = MASTFile("test_control/submitlist")
        self.assertEquals(mysubmit.data[0], "%s\n" % ingdir)
        #self.testclass.run_singlerun(mode='serial')

    def test_run_neb_subfolders(self):
        #raise SkipTest
        ingdir="%s/writedir/neb_labelinit-labelfin_stat" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_neb")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_neb")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        kdict['mast_kpoints'] = [2,2,2,"M"]
        kdict['mast_xc'] = 'pw91'
        neblines = list()
        neblines.append(["Cr","0.0 0.9 0.8","0.0 0.8 0.7"])
        neblines.append(["Cr","0.4 0.2 0.1","0.3 0.3 0.2"])
        neblines.append(["Cr","0.29 0.05 0.05","0.01 0.01 0.98"])
        neblines.append(["Ni","0.61 0.99 0.98","0.25 0.01 0.97"])
        kdict['mast_neb_settings']=dict()
        kdict['mast_neb_settings']['lines']=neblines
        kdict['mast_neb_settings']['images']=3
        str_00 = MASTFile("files/POSCAR_00")
        str_00.to_file("%s/parent_structure_labelinit" % ingdir) 
        str_04 = MASTFile("files/POSCAR_04")
        str_04.to_file("%s/parent_structure_labelfin" % ingdir) 
        str_01 = MASTFile("files/POSCAR_01")
        str_01.to_file("%s/parent_structure_labelinit-labelfin_01" % ingdir) 
        str_02 = MASTFile("files/POSCAR_02")
        str_02.to_file("%s/parent_structure_labelinit-labelfin_02" % ingdir) 
        str_03 = MASTFile("files/POSCAR_03")
        str_03.to_file("%s/parent_structure_labelinit-labelfin_03" % ingdir) 
        en_00 = MASTFile("files/OSZICAR_00")
        en_00.to_file("%s/parent_energy_labelinit" % ingdir) 
        en_04 = MASTFile("files/OSZICAR_04")
        en_04.to_file("%s/parent_energy_labelfin" % ingdir) 
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        mywr = ChopIngredient(name=ingdir, program_keys = kdict, structure=my_structure)
        mywr.write_neb_subfolders()
        myri = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_structure)
        myri.run_neb_subfolders()
        mysubmit = MASTFile("test_control/submitlist")
        myri.checker.keywords['name'] = "%s/00" % ingdir
        self.assertFalse(myri.checker.is_ready_to_run()) #do not run endpoints again
        myri.checker.keywords['name'] = "%s/01" % ingdir
        self.assertTrue(myri.checker.is_ready_to_run())
        myri.checker.keywords['name'] = "%s/02" % ingdir
        self.assertTrue(myri.checker.is_ready_to_run())
        myri.checker.keywords['name'] = "%s/03" % ingdir
        self.assertTrue(myri.checker.is_ready_to_run())
        myri.checker.keywords['name'] = "%s/04" % ingdir
        self.assertFalse(myri.checker.is_ready_to_run()) #do not run endpoints again
        self.assertEquals(mysubmit.data[0], "%s/01\n" % ingdir)
        self.assertEquals(mysubmit.data[1], "%s/02\n" % ingdir)
        self.assertEquals(mysubmit.data[2], "%s/03\n" % ingdir)
        #self.testclass.run_neb_subfolders()

    def test_run_subfolders(self):
        #raise SkipTest
        ingdir="%s/writedir/single_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        kdict['mast_kpoints'] = [2,2,2,"M"]
        kdict['mast_xc'] = 'pw91'
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        for subfolder in ['sub1','sub2','sub3','sub4']:
            subname = "%s/%s" % (ingdir, subfolder)
            os.mkdir(subname)
            shutil.copy("files/metadata_single","%s/metadata.txt" % subname)
            mywr = ChopIngredient(name=subname, program_keys = kdict, structure=my_structure)
            mywr.write_singlerun()
        myri = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_structure)
        myri.run_subfolders()
        self.assertFalse(myri.checker.is_ready_to_run())
        for subfolder in ['sub1','sub2','sub3','sub4']:
            subname = "%s/%s" % (ingdir, subfolder)
            myri.checker.keywords['name'] = subname
            self.assertTrue(myri.checker.is_ready_to_run())
        mysubmit = MASTFile("test_control/submitlist")
        self.assertEquals(mysubmit.data[0], "%s/sub1\n" % ingdir)
        self.assertEquals(mysubmit.data[1], "%s/sub2\n" % ingdir)
        self.assertEquals(mysubmit.data[2], "%s/sub3\n" % ingdir)
        self.assertEquals(mysubmit.data[3], "%s/sub4\n" % ingdir)
        #self.testclass.run_subfolders()

    def test_run_defect(self):
        #raise SkipTest
        ingdir="%s/writedir/single_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.data.append("defect_label = label1\n")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['label1']=dict()
        kdict['label1']['subdefect1']=dict()
        kdict['label1']['subdefect1']['symbol']='Cr'
        kdict['label1']['subdefect1']['type']='interstitial'
        kdict['label1']['subdefect1']['coordinates']=np.array([0.8, 0.7, 0.6])
        kdict['label1']['subdefect2']=dict()
        kdict['label1']['subdefect2']['symbol']='Sr'
        kdict['label1']['subdefect2']['type']='antisite'
        kdict['label1']['subdefect2']['coordinates']=np.array([0.5,0.5,0.0])
        kdict['label1']['subdefect3']=dict()
        kdict['label1']['subdefect3']['symbol']='Cr'
        kdict['label1']['subdefect3']['type']='interstitial'
        kdict['label1']['subdefect3']['coordinates']=np.array([0.3,0.3,0.2])
        kdict['label1']['subdefect4']=dict()
        kdict['label1']['subdefect4']['symbol']='Fe'
        kdict['label1']['subdefect4']['type']='substitution'
        kdict['label1']['subdefect4']['coordinates']=np.array([0.25,0.75,0.75])
        kdict['label1']['subdefect5']=dict()
        kdict['label1']['subdefect5']['symbol']='O'
        kdict['label1']['subdefect5']['type']='vacancy'
        kdict['label1']['subdefect5']['coordinates']=np.array([0.5,0.25,0.75])
        kdict['label1']['subdefect6']=dict()
        kdict['label1']['subdefect6']['symbol']='Fe'
        kdict['label1']['subdefect6']['type']='substitution'
        kdict['label1']['subdefect6']['coordinates']=np.array([0.25,0.25,0.25])
        kdict['label1']['subdefect7']=dict()
        kdict['label1']['subdefect7']['symbol']='La'
        kdict['label1']['subdefect7']['type']='vacancy'
        kdict['label1']['subdefect7']['coordinates']=np.array([0,0,0.5])
        kdict['label1']['subdefect8']=dict()
        kdict['label1']['subdefect8']['symbol']='Ni'
        kdict['label1']['subdefect8']['type']='interstitial'
        kdict['label1']['subdefect8']['coordinates']=np.array([0.4,0.1,0.3])
        kdict['label1']['coord_type'] = 'fractional'
        kdict['label1']['threshold'] = 0.01
        kdict['label1']['charge'] = '2'
        ddict=dict()
        ddict['mast_defect_settings']=dict()
        ddict['mast_defect_settings'].update(kdict['label1']) #single defect grouping
        ddict['mast_program'] = 'vasp'
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/POSCAR_perfect").structure
        myperf = MASTFile("files/POSCAR_perfect")
        myperf.to_file("%s/POSCAR" % ingdir)
        myri = ChopIngredient(name=ingdir,program_keys=ddict, structure=my_structure)
        myri.run_defect()
        #
        #defect = kdict['label1']
        #base_structure = my_structure.copy()
        #from MAST.ingredients.pmgextend.structure_extensions import StructureExtensions
        #for key in defect:
        #    if 'subdefect' in key:
        #        subdefect = defect[key]
        #        sxtend = StructureExtensions(struc_work1=base_structure)
        #        base_structure = sxtend.induce_defect(subdefect, defect['coord_type'], defect['threshold'])
        #        print base_structure
        #    else:
        #        pass
        #return
        #
        #
        my_defected = pymatgen.io.vaspio.Poscar.from_file("%s/CONTCAR" % ingdir).structure.get_sorted_structure()
        defected_compare = pymatgen.io.vaspio.Poscar.from_file("files/POSCAR_multi").structure.get_sorted_structure()
        self.assertEquals(my_defected, defected_compare)
        self.assertFalse(os.path.isfile("test_control/submitlist"))
        #self.testclass.run_defect()

    def test_run_strain(self):
        #raise SkipTest
        ingdir="%s/writedir/single_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        kdict['mast_strain']=" 0.98 0.92 1.03 \n"
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/POSCAR_perfect").structure
        myunstrained = MASTFile("files/POSCAR_unstrained")
        myunstrained.to_file("%s/POSCAR" % ingdir)
        myri = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_structure)
        myri.run_strain()
        my_strained = pymatgen.io.vaspio.Poscar.from_file("%s/CONTCAR" % ingdir).structure
        strained_compare = pymatgen.io.vaspio.Poscar.from_file("files/POSCAR_strained").structure
        self.assertEquals(my_strained, strained_compare)
        self.assertFalse(os.path.isfile("test_control/submitlist"))
        #self.testclass.run_strain()

    def test_run_scale_defect(self):
        #raise SkipTest
        ingdir="%s/writedir/single_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.data.append("defect_label = label1\n")
        metad.data.append("scaling_size = 2x2x2\n")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['label1']=dict()
        kdict['label1']['subdefect1']=dict()
        kdict['label1']['subdefect1']['symbol']='Cr'
        kdict['label1']['subdefect1']['type']='interstitial'
        kdict['label1']['subdefect1']['coordinates']=np.array([0.8, 0.7, 0.6])
        kdict['label1']['subdefect2']=dict()
        kdict['label1']['subdefect2']['symbol']='Sr'
        kdict['label1']['subdefect2']['type']='antisite'
        kdict['label1']['subdefect2']['coordinates']=np.array([0.5,0.5,0.0])
        kdict['label1']['subdefect3']=dict()
        kdict['label1']['subdefect3']['symbol']='Cr'
        kdict['label1']['subdefect3']['type']='interstitial'
        kdict['label1']['subdefect3']['coordinates']=np.array([0.3,0.3,0.2])
        kdict['label1']['subdefect4']=dict()
        kdict['label1']['subdefect4']['symbol']='Fe'
        kdict['label1']['subdefect4']['type']='substitution'
        kdict['label1']['subdefect4']['coordinates']=np.array([0.25,0.75,0.75])
        kdict['label1']['subdefect5']=dict()
        kdict['label1']['subdefect5']['symbol']='O'
        kdict['label1']['subdefect5']['type']='vacancy'
        kdict['label1']['subdefect5']['coordinates']=np.array([0.5,0.25,0.75])
        kdict['label1']['subdefect6']=dict()
        kdict['label1']['subdefect6']['symbol']='Fe'
        kdict['label1']['subdefect6']['type']='substitution'
        kdict['label1']['subdefect6']['coordinates']=np.array([0.25,0.25,0.25])
        kdict['label1']['subdefect7']=dict()
        kdict['label1']['subdefect7']['symbol']='La'
        kdict['label1']['subdefect7']['type']='vacancy'
        kdict['label1']['subdefect7']['coordinates']=np.array([0,0,0.5])
        kdict['label1']['subdefect8']=dict()
        kdict['label1']['subdefect8']['symbol']='Ni'
        kdict['label1']['subdefect8']['type']='interstitial'
        kdict['label1']['subdefect8']['coordinates']=np.array([0.4,0.1,0.3])
        kdict['label1']['coord_type'] = 'fractional'
        kdict['label1']['threshold'] = 0.01
        kdict['label1']['charge'] = '2'
        mdict=dict()
        mdict['mast_program'] = 'vasp'
        mdict['mast_defect_settings']=dict()
        mdict['mast_defect_settings'].update(kdict['label1'])
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/POSCAR_perfect").structure
        myperf = MASTFile("files/POSCAR_perfect")
        myperf.to_file("%s/POSCAR" % ingdir)
        myri = ChopIngredient(name=ingdir,program_keys=mdict, structure=my_structure)
        myri.run_scale()
        #copy contcar to poscar
        os.rename("%s/CONTCAR" % ingdir, "%s/POSCAR" % ingdir)
        myri.run_defect()
        my_defected = pymatgen.io.vaspio.Poscar.from_file("%s/CONTCAR" % ingdir).structure.get_sorted_structure()
        defected_compare = pymatgen.io.vaspio.Poscar.from_file("files/POSCAR_scaled_defected").structure.get_sorted_structure()
        self.assertEquals(my_defected, defected_compare)
    def test_run_scale(self):
        #raise SkipTest
        ingdir="%s/writedir/single_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.data.append("defect_label = label1\n")
        metad.data.append("scaling_size = 2x2x2\n")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/POSCAR_perfect").structure
        myperf = MASTFile("files/POSCAR_perfect")
        myperf.to_file("%s/POSCAR" % ingdir)
        myri = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_structure)
        myri.run_scale()
        my_scaled = pymatgen.io.vaspio.Poscar.from_file("%s/CONTCAR" % ingdir).structure.get_sorted_structure()
        scaled_compare = pymatgen.io.vaspio.Poscar.from_file("files/POSCAR_scaled").structure.get_sorted_structure()
        self.assertEquals(my_scaled, scaled_compare)



from operator import attrgetter

def get_best(pop, nkeep):
    """Function for sorting population by fitness attributes"""
    pop = sorted(pop, key=attrgetter('fitness')) 
    pop = pop[0:nkeep]
    return pop



"""Tests for Recipeplan"""

from MAST.recipe.recipeplan import RecipePlan

import unittest
from unittest import SkipTest
import os
import time
import MAST
import pymatgen
import shutil
from MAST.utility import dirutil
from MAST.utility import MASTFile
from MAST.utility import MASTError
testname="recipeplan_test"
testdir = dirutil.get_test_dir(testname)
old_control = os.getenv("MAST_CONTROL")
old_scratch = os.getenv("MAST_SCRATCH")
old_recipe = os.getenv("MAST_RECIPE_PATH")

class TestRecipeplan(unittest.TestCase):

    def setUp(self):
        os.chdir(testdir)
        if not os.path.isdir("test_control"):
            os.mkdir("test_control")
        if not os.path.isdir("recipedir"):
            os.mkdir("recipedir")
        if not os.path.isdir("recipedir/ing2a"):
            os.mkdir("recipedir/ing2a")
        if not os.path.isdir("recipedir/ing2b"):
            os.mkdir("recipedir/ing2b")
        if not os.path.isdir("recipedir/ing3"):
            os.mkdir("recipedir/ing3")
        if not os.path.isfile("test_control/set_platform"):
            shutil.copy(old_control + "/set_platform", "test_control/set_platform")
        if not os.path.isdir("test_control/programkeys"):
            shutil.copytree(old_control + "/programkeys", "test_control/programkeys")
        os.environ['MAST_CONTROL'] = testdir + "/test_control"
        os.environ['MAST_RECIPE_PATH'] = testdir
        os.environ['MAST_SCRATCH'] = testdir

    def tearDown(self):
        removelist=list()
        removelist.append("recipedir/status.txt")
        removelist.append("recipedir/ing2b/INCAR")
        removelist.append("recipedir/ing2b/POSCAR")
        removelist.append("recipedir/ing2b/POTCAR")
        removelist.append("recipedir/ing2b/KPOINTS")
        removelist.append("recipedir/ing2b/submit.sh")
        removelist.append("recipedir/ing2b/CHGCAR")
        removelist.append("recipedir/ing2b/WAVECAR")
        removelist.append("recipedir/ing2a/POSCAR")
        removelist.append("recipedir/printed.txt")
        removelist.append("test_control/submitlist")
        removelist.append("test_control/set_platform")
        for myfile in removelist:
            try:
                os.remove(myfile)
            except OSError:
                pass
        os.environ['MAST_CONTROL'] = old_control
        os.environ['MAST_RECIPE_PATH'] = old_recipe
        os.environ['MAST_SCRATCH'] = old_scratch

    def test___init__(self):
        rp = RecipePlan("test_recipe","recipedir")
        self.assertEquals(rp.working_directory,"recipedir")
        self.assertEquals(rp.status,"I")
        self.assertEquals(rp.name,"test_recipe")
        #raise SkipTest
        #self.testclass.__init__(name, working_directory)

    def test_write_ingredient(self):
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("recipedir/metadata.txt")
        #metad = MASTFile("files/metadata_single")
        #metad.to_file("%s/metadata.txt" % ingdir)
        rp = RecipePlan("test_recipe","recipedir")
        rp.ingredients['ing2b'] = "I"
        kdict=dict()
        kdict['mast_program']='vasp'
        kdict['mast_xc']='pw91'
        kdict['mast_kpoints']=[1,2,3,"G"]
        rp.ingred_input_options['ing2b']=dict()
        rp.ingred_input_options['ing2b']['name']="recipedir/ing2b"
        rp.ingred_input_options['ing2b']['program_keys']=kdict
        rp.ingred_input_options['ing2b']['structure']=pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        rp.write_methods['ing2b']=[['write_singlerun']]
        rp.write_ingredient('ing2b')
        self.assertTrue(os.path.isfile('recipedir/ing2b/INCAR'))
        self.assertTrue(os.path.isfile('recipedir/ing2b/POSCAR'))
        self.assertTrue(os.path.isfile('recipedir/ing2b/POTCAR'))
        self.assertTrue(os.path.isfile('recipedir/ing2b/KPOINTS'))
        self.assertTrue(os.path.isfile('recipedir/ing2b/submit.sh'))
        #self.testclass.write_ingredient(iname)

    def test_complete_ingredient(self):
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("recipedir/metadata.txt")
        #metad = MASTFile("files/metadata_single")
        #metad.to_file("%s/metadata.txt" % ingdir)
        rp = RecipePlan("test_recipe","recipedir")
        rp.ingredients['ing1'] = "I"
        kdict=dict()
        kdict['mast_program']='vasp'
        kdict['mast_xc']='pw91'
        kdict['mast_kpoints']=[1,2,3,"G"]
        rp.ingred_input_options['ing1']=dict()
        rp.ingred_input_options['ing1']['name']="%s/recipedir/ing1" % testdir
        rp.ingred_input_options['ing1']['program_keys']=kdict
        rp.ingred_input_options['ing1']['structure']=pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        rp.complete_methods['ing1']=[['complete_singlerun']]
        self.assertTrue(rp.complete_ingredient('ing1'))
        #self.testclass.complete_ingredient(iname)

    def test_ready_ingredient(self):
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("recipedir/metadata.txt")
        #metad = MASTFile("files/metadata_single")
        #metad.to_file("%s/metadata.txt" % ingdir)
        rp = RecipePlan("test_recipe","recipedir")
        rp.ingredients['ing2b'] = "I"
        kdict=dict()
        kdict['mast_program']='vasp'
        kdict['mast_xc']='pw91'
        kdict['mast_kpoints']=[1,2,3,"G"]
        rp.ingred_input_options['ing2b']=dict()
        rp.ingred_input_options['ing2b']['name']="recipedir/ing2b"
        rp.ingred_input_options['ing2b']['program_keys']=kdict
        rp.ingred_input_options['ing2b']['structure']=pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        rp.write_methods['ing2b']=[['write_singlerun']]
        rp.write_ingredient('ing2b')
        rp.ready_methods['ing2b']=[['ready_singlerun']]
        self.assertTrue(rp.ready_ingredient('ing2b'))
        #self.testclass.ready_ingredient(iname)

    def test_run_ingredient(self):
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("recipedir/metadata.txt")
        #metad = MASTFile("files/metadata_single")
        #metad.to_file("%s/metadata.txt" % ingdir)
        rp = RecipePlan("test_recipe","recipedir")
        rp.ingredients['ing2b'] = "I"
        kdict=dict()
        kdict['mast_program']='vasp'
        kdict['mast_xc']='pw91'
        kdict['mast_kpoints']=[1,2,3,"G"]
        rp.ingred_input_options['ing2b']=dict()
        rp.ingred_input_options['ing2b']['name']="recipedir/ing2b"
        rp.ingred_input_options['ing2b']['program_keys']=kdict
        rp.ingred_input_options['ing2b']['structure']=pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        rp.write_methods['ing2b']=[['write_singlerun']]
        rp.write_ingredient('ing2b')
        rp.run_methods['ing2b']=[['run_singlerun']]
        rp.run_ingredient('ing2b')
        mysubmit = MASTFile("test_control/submitlist")
        self.assertEquals(mysubmit.data[0],"recipedir/ing2b\n")
        #self.testclass.run_ingredient(iname)

    def test_update_children(self):
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("recipedir/metadata.txt")
        #metad = MASTFile("files/metadata_single")
        #metad.to_file("%s/metadata.txt" % ingdir)
        rp = RecipePlan("test_recipe","%s/recipedir" % testdir)
        rp.ingredients['ing1'] = "I"
        kdict=dict()
        kdict['mast_program']='vasp'
        kdict['mast_xc']='pw91'
        kdict['mast_kpoints']=[1,2,3,"G"]
        rp.ingred_input_options['ing1']=dict()
        rp.ingred_input_options['ing1']['name']="%s/ing1" % rp.working_directory
        rp.ingred_input_options['ing1']['program_keys']=kdict
        rp.ingred_input_options['ing1']['structure']=pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        rp.update_methods['ing1']=dict()
        rp.update_methods['ing1']['ing2a']=[['give_structure']]
        rp.update_methods['ing1']['ing2b']=[['give_structure_and_restart_files']]
        rp.update_children('ing1')
        self.assertTrue(os.path.isfile("recipedir/ing2a/POSCAR"))
        self.assertTrue(os.path.isfile("recipedir/ing2b/POSCAR"))
        #CHGCAR softlink only sent to second child
        self.assertFalse(os.path.exists("recipedir/ing2a/CHGCAR"))
        self.assertTrue(os.path.exists("recipedir/ing2b/CHGCAR"))
        #self.testclass.update_children(iname)

    def test_fast_forward_check_complete(self):
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("recipedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("recipedir/ing1/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.data.append("defect_label = labela\n")
        metad.to_file("recipedir/ing2a/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.data.append("defect_label = labelb\n")
        metad.to_file("recipedir/ing2b/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("recipedir/ing3/metadata.txt")
        rp = RecipePlan("test_recipe","recipedir")
        rp.ingredients['ing1'] = "I"
        rp.ingredients['ing2a'] = "I"
        rp.ingredients['ing2b'] = "I"
        rp.ingredients['ing3'] = "I"
        kdict=dict()
        kdict['mast_program']='vasp'
        kdict['mast_xc']='pw91'
        kdict['mast_kpoints']=[1,2,3,"G"]
        my_struc = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        rp.ingred_input_options['ing1']=dict()
        rp.ingred_input_options['ing1']['name']="%s/recipedir/ing1" % testdir
        rp.ingred_input_options['ing1']['program_keys']=kdict
        rp.ingred_input_options['ing1']['structure']=my_struc
        rp.complete_methods['ing1']=[['complete_singlerun']]
        rp.update_methods['ing1']=dict()
        rp.update_methods['ing1']['ing2a']=[['give_structure']]
        rp.update_methods['ing1']['ing2b']=[['give_structure']]
        rp.ingred_input_options['ing2a']=dict()
        rp.ingred_input_options['ing2a']['name']="%s/recipedir/ing2a" % testdir
        rp.ingred_input_options['ing2a']['program_keys']=kdict
        rp.ingred_input_options['ing2a']['structure']=my_struc
        rp.complete_methods['ing2a']=[['complete_singlerun']]
        rp.ready_methods['ing2a']=[['ready_structure']]
        rp.ingred_input_options['ing2b']=dict()
        rp.ingred_input_options['ing2b']['name']="%s/recipedir/ing2b" % testdir
        rp.ingred_input_options['ing2b']['program_keys']=kdict
        rp.ingred_input_options['ing2b']['structure']=my_struc
        rp.complete_methods['ing2b']=[['complete_singlerun']]
        rp.ready_methods['ing2b']=[['ready_structure']]
        rp.ingred_input_options['ing3']=dict()
        rp.ingred_input_options['ing3']['name']="%s/recipedir/ing3" % testdir
        rp.ingred_input_options['ing3']['program_keys']=kdict
        rp.ingred_input_options['ing3']['structure']=my_struc
        rp.complete_methods['ing3']=[['complete_singlerun']]
        rp.ready_methods['ing3']=[['ready_structure']]
        rp.fast_forward_check_complete()
        #self.assertTrue(rp.complete_ingredient('ing1'))
        self.assertEquals(rp.ingredients, {'ing1':'C','ing2a':'I','ing2b':'I','ing3':'I'})
        self.assertTrue(rp.ready_ingredient('ing2a'))
        self.assertTrue(rp.ready_ingredient('ing2b'))
        #self.testclass.fast_forward_check_complete()

    def test_check_if_have_parents(self):
        rp = RecipePlan("test_recipe","recipedir")
        rp.ingredients['ing1'] = "I"
        rp.ingredients['ing2a'] = "I"
        rp.ingredients['ing2b'] = "I"
        rp.ingredients['ing3'] = "I"
        rp.parents_to_check['ing3']=['ing2a','ing2b']
        rp.parents_to_check['ing2a']=['ing1']
        rp.parents_to_check['ing2b']=[]
        rp.parents_to_check['ing1']=[]
        rp.check_if_have_parents()
        self.assertEquals(rp.ingredients,{'ing1':'S','ing2a':'W','ing2b':'S','ing3':'W'})
        #self.testclass.check_if_have_parents()

    def test_check_if_ready_to_proceed_are_complete(self):
        metad = MASTFile("files/metadata_single")
        metad.to_file("recipedir/ing1/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("recipedir/ing2a/metadata.txt")
        rp = RecipePlan("test_recipe","recipedir")
        rp.ingredients['ing1'] = "P"
        rp.ingredients['ing2a'] = "I"
        rp.ingredients['ing2b'] = "I"
        rp.ingredients['ing3'] = "I"
        kdict=dict()
        kdict['mast_program']='vasp'
        kdict['mast_xc']='pw91'
        kdict['mast_kpoints']=[1,2,3,"G"]
        my_struc = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        rp.ingred_input_options['ing1']=dict()
        rp.ingred_input_options['ing1']['name']="%s/recipedir/ing1" % testdir
        rp.ingred_input_options['ing1']['program_keys']=kdict
        rp.ingred_input_options['ing1']['structure']=my_struc
        rp.complete_methods['ing1']=[['complete_singlerun']]
        rp.update_methods['ing1']=dict()
        rp.update_methods['ing1']['ing2a']=[['give_structure']]
        rp.update_methods['ing1']['ing2b']=[['give_structure']]
        rp.ingred_input_options['ing2a']=dict()
        rp.ingred_input_options['ing2a']['name']="%s/recipedir/ing2a" % testdir
        rp.ingred_input_options['ing2a']['program_keys']=kdict
        rp.ingred_input_options['ing2a']['structure']=my_struc
        rp.complete_methods['ing2a']=[['complete_singlerun']]
        rp.ready_methods['ing2a']=[['ready_structure']]
        rp.check_if_ready_to_proceed_are_complete()
        self.assertTrue(rp.ready_ingredient('ing2a'))
        self.assertEquals
        self.assertEquals(rp.ingredients,{'ing1':'C','ing2a':'I','ing2b':'I','ing3':'I'})
        #self.testclass.check_if_ready_to_proceed_are_complete()

    def test_check_if_parents_are_complete(self):
        rp = RecipePlan("test_recipe","recipedir")
        rp.ingredients['ing1'] = "C"
        rp.ingredients['ing2a'] = "W"
        rp.ingredients['ing2b'] = "I"
        rp.ingredients['ing3'] = "I"
        rp.parents_to_check['ing3']=['ing2a','ing2b']
        rp.parents_to_check['ing2a']=['ing1']
        rp.parents_to_check['ing2b']=[]
        rp.parents_to_check['ing1']=[]
        rp.check_if_parents_are_complete()
        self.assertEquals(rp.ingredients,{'ing1':'C','ing2a':'S','ing2b':'I','ing3':'I'})
        #self.testclass.check_if_parents_are_complete()

    def test_run_staged_ingredients(self):
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("recipedir/metadata.txt")
        #metad = MASTFile("files/metadata_single")
        #metad.to_file("%s/metadata.txt" % ingdir)
        rp = RecipePlan("test_recipe","recipedir")
        rp.ingredients['ing1']="C"
        rp.ingredients['ing2a'] = "W"
        rp.ingredients['ing2b'] = "S"
        rp.ingredients['ing3'] = "W"
        kdict=dict()
        kdict['mast_program']='vasp'
        kdict['mast_xc']='pw91'
        kdict['mast_kpoints']=[1,2,3,"G"]
        rp.ingred_input_options['ing2b']=dict()
        rp.ingred_input_options['ing2b']['name']="recipedir/ing2b"
        rp.ingred_input_options['ing2b']['program_keys']=kdict
        rp.ingred_input_options['ing2b']['structure']=pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        rp.write_methods['ing2b']=[['write_singlerun']]
        rp.write_ingredient('ing2b')
        rp.ready_methods['ing2b']=[['ready_singlerun']]
        rp.run_methods['ing2b']=[['run_singlerun']]
        rp.complete_methods['ing2b']=[['complete_singlerun']]
        rp.run_staged_ingredients()
        mysubmit = MASTFile("test_control/submitlist")
        self.assertEquals(mysubmit.data[0],"recipedir/ing2b\n")
        self.assertEquals(rp.ingredients,{'ing1':'C','ing2a':'W','ing2b':'P','ing3':'W'})
        #self.testclass.run_staged_ingredients()

    def test_check_recipe_status(self):
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("recipedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("recipedir/ing1/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.data.append("defect_label = labela\n")
        metad.to_file("recipedir/ing2a/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.data.append("defect_label = labelb\n")
        metad.to_file("recipedir/ing2b/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("recipedir/ing3/metadata.txt")
        rp = RecipePlan("test_recipe","%s/recipedir" % testdir)
        rp.ingredients['ing1'] = "I"
        rp.ingredients['ing2a'] = "I"
        rp.ingredients['ing2b'] = "I"
        rp.ingredients['ing3'] = "I"
        kdict=dict()
        kdict['mast_program']='vasp'
        kdict['mast_xc']='pw91'
        kdict['mast_kpoints']=[1,2,3,"G"]
        my_struc = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        rp.ingred_input_options['ing1']=dict()
        rp.ingred_input_options['ing1']['name']="%s/recipedir/ing1" % testdir
        rp.ingred_input_options['ing1']['program_keys']=kdict
        rp.ingred_input_options['ing1']['structure']=my_struc
        rp.complete_methods['ing1']=[['complete_singlerun']]
        rp.run_methods['ing1']=[['run_singlerun']]
        rp.update_methods['ing1']=dict()
        rp.update_methods['ing1']['ing2a']=[['give_structure']]
        rp.update_methods['ing1']['ing2b']=[['give_structure']]
        rp.ingred_input_options['ing2a']=dict()
        rp.ingred_input_options['ing2a']['name']="%s/recipedir/ing2a" % testdir
        rp.ingred_input_options['ing2a']['program_keys']=kdict
        rp.ingred_input_options['ing2a']['structure']=my_struc
        rp.complete_methods['ing2a']=[['complete_singlerun']]
        rp.ready_methods['ing2a']=[['ready_structure']]
        rp.run_methods['ing2a']=[['run_singlerun']]
        rp.ingred_input_options['ing2b']=dict()
        rp.ingred_input_options['ing2b']['name']="%s/recipedir/ing2b" % testdir
        rp.ingred_input_options['ing2b']['program_keys']=kdict
        rp.ingred_input_options['ing2b']['structure']=my_struc
        rp.complete_methods['ing2b']=[['complete_singlerun']]
        rp.ready_methods['ing2b']=[['ready_structure']]
        rp.run_methods['ing2b']=[['run_singlerun']]
        rp.ingred_input_options['ing3']=dict()
        rp.ingred_input_options['ing3']['name']="%s/recipedir/ing3" % testdir
        rp.ingred_input_options['ing3']['program_keys']=kdict
        rp.ingred_input_options['ing3']['structure']=my_struc
        rp.complete_methods['ing3']=[['complete_singlerun']]
        rp.ready_methods['ing3']=[['ready_structure']]
        rp.run_methods['ing3']=[['run_singlerun']]
        rp.parents_to_check['ing3']=['ing2a','ing2b']
        rp.parents_to_check['ing2a']=['ing1']
        rp.parents_to_check['ing2b']=[]
        rp.parents_to_check['ing1']=[]
        rp.check_recipe_status()
        mystatus = MASTFile("%s/recipedir/status.txt" % testdir)
        status_compare = MASTFile("%s/files/status_current.txt" % testdir)
        self.assertEqual(mystatus.data, status_compare.data)
        
        #self.testclass.check_recipe_status(verbose=1)

    def test_print_status(self):
        rp = RecipePlan("test_recipe","%s/recipedir" % testdir)
        rp.ingredients['ing1'] = "S"
        rp.ingredients['ing2a'] = "W"
        rp.ingredients['ing2b'] = "C"
        rp.ingredients['ing3'] = "P"
        rp.print_status()
        mystatus = MASTFile("recipedir/status.txt")
        status_compare = MASTFile("files/status_artificial.txt")
        self.assertEqual(mystatus.data, status_compare.data)
        #self.testclass.print_status(verbose=1)

    def test___repr__(self):
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("recipedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("recipedir/ing1/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.data.append("defect_label = labela\n")
        metad.to_file("recipedir/ing2a/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.data.append("defect_label = labelb\n")
        metad.to_file("recipedir/ing2b/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("recipedir/ing3/metadata.txt")
        rp = RecipePlan("test_recipe","recipedir")
        rp.ingredients['ing1'] = "I"
        rp.ingredients['ing2a'] = "I"
        rp.ingredients['ing2b'] = "I"
        rp.ingredients['ing3'] = "I"
        kdict=dict()
        kdict['mast_program']='vasp'
        kdict['mast_xc']='pw91'
        kdict['mast_kpoints']=[1,2,3,"G"]
        my_struc = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        rp.ingred_input_options['ing1']=dict()
        rp.ingred_input_options['ing1']['name']="recipedir/ing1"
        rp.ingred_input_options['ing1']['program_keys']=kdict
        rp.ingred_input_options['ing1']['structure']=my_struc
        rp.complete_methods['ing1']=[['complete_singlerun']]
        rp.run_methods['ing1']=[['run_singlerun']]
        rp.ready_methods['ing1']=[['ready_singlerun']]
        rp.write_methods['ing1']=[['write_singlerun']]
        rp.update_methods['ing1']=dict()
        rp.update_methods['ing1']['ing2a']=[['give_structure']]
        rp.update_methods['ing1']['ing2b']=[['give_structure']]
        rp.ingred_input_options['ing2a']=dict()
        rp.ingred_input_options['ing2a']['name']="recipedir/ing2a"
        rp.ingred_input_options['ing2a']['program_keys']=kdict
        rp.ingred_input_options['ing2a']['structure']=my_struc
        rp.complete_methods['ing2a']=[['complete_singlerun']]
        rp.ready_methods['ing2a']=[['ready_structure']]
        rp.run_methods['ing2a']=[['run_singlerun']]
        rp.write_methods['ing2a']=[['write_singlerun']]
        rp.update_methods['ing2a']=dict()
        rp.update_methods['ing2a']['ing3']=[['give_structure_and_restart_files']]
        rp.ingred_input_options['ing2b']=dict()
        rp.ingred_input_options['ing2b']['name']="recipedir/ing2b"
        rp.ingred_input_options['ing2b']['program_keys']=kdict
        rp.ingred_input_options['ing2b']['structure']=my_struc
        rp.complete_methods['ing2b']=[['complete_singlerun']]
        rp.ready_methods['ing2b']=[['ready_structure']]
        rp.run_methods['ing2b']=[['run_singlerun']]
        rp.write_methods['ing2b']=[['write_singlerun']]
        rp.update_methods['ing2b']=dict()
        rp.update_methods['ing2b']['ing3']=[['give_structure_and_restart_files']]
        rp.ingred_input_options['ing3']=dict()
        rp.ingred_input_options['ing3']['name']="recipedir/ing3"
        rp.ingred_input_options['ing3']['program_keys']=kdict
        rp.ingred_input_options['ing3']['structure']=my_struc
        rp.complete_methods['ing3']=[['complete_singlerun']]
        rp.ready_methods['ing3']=[['ready_structure']]
        rp.run_methods['ing3']=[['run_singlerun']]
        rp.write_methods['ing3']=[['write_singlerun']]
        rp.update_methods['ing3']=dict()
        rp.parents_to_check['ing3']=['ing2a','ing2b']
        rp.parents_to_check['ing2a']=['ing1']
        rp.parents_to_check['ing2b']=[]
        rp.parents_to_check['ing1']=[]
        mylines=rp.__repr__()
        printed =MASTFile()
        printed.data = mylines
        printed.to_file("recipedir/printed.txt")
        compare_print=MASTFile("files/printout.txt")
        printedread = MASTFile("recipedir/printed.txt")
        self.assertEqual(printedread.data, compare_print.data)

        #self.testclass.__repr__()

    def test_get_statuses_from_file(self):
        rp = RecipePlan("test_recipe","recipedir")
        mystatus = MASTFile("files/status_random.txt")
        self.assertRaises(MASTError, rp.get_statuses_from_file)
        mystatus.to_file("recipedir/status.txt")
        rp.ingredients['ing1']="I"
        rp.ingredients['ing2a']="I"
        rp.ingredients['ing2b']="I"
        self.assertRaises(MASTError,rp.get_statuses_from_file)
        rp.ingredients['ing3']="I"
        rp.get_statuses_from_file()
        statusdict=dict()
        statusdict={'ing1':'alpha','ing2a':'beta','ing2b':'gamma','ing3':'delta'}
        self.assertEquals(rp.ingredients, statusdict)
        #self.testclass.get_statuses_from_file()




import os
import time
import unittest
from unittest import SkipTest
import filecmp
from filecmp import dircmp
import MAST
from MAST.ingredients.checker.vaspchecker import VaspChecker
import shutil
import pymatgen
import numpy as np
from MAST.utility import MASTError
from MAST.utility import dirutil
from MAST.utility import MASTFile
testname ="checker_test_vasp"
#oldcontrol = os.getenv("MAST_CONTROL")
#oldrecipe = os.getenv("MAST_RECIPE_PATH")
#oldscratch = os.getenv("MAST_SCRATCH")
#print "Old directories:"
#print oldcontrol
#print oldrecipe
#print oldscratch
testdir = dirutil.get_test_dir(testname)


class TestVaspChecker(unittest.TestCase):
    """Test Vasp Checker
    """
    def setUp(self):
        os.chdir(testdir)
        if not os.path.exists("childdir"):
            os.mkdir("childdir")
        shutil.copy("files/metadata.txt","childdir")

    def tearDown(self):
        for fname in ["POSCAR","XDATCAR","DYNMAT","OSZICAR","DYNMAT_combined","KPOINTS","POTCAR","INCAR","WAVECAR","CHGCAR","POSCAR_no_sd","XDATCAR_combined","CONTCAR","metadata.txt"]:
            try:
                os.remove("childdir/%s" % fname)
            except OSError:
                pass
        os.rmdir("childdir")

    def test_get_structure_from_file(self):
        myvc = VaspChecker(name = "structure")
        getstr = myvc.get_structure_from_file("structure/POSCAR_perfect")
        mystr = pymatgen.io.vaspio.Poscar.from_file("structure/POSCAR_perfect").structure
        self.assertEqual(mystr, getstr)

    def test_get_initial_structure(self):
        myvc = VaspChecker(name = "structure")
        nodir = myvc.get_initial_structure_from_directory()
        compare_nodir = pymatgen.io.vaspio.Poscar.from_file("structure/POSCAR").structure
        self.assertEqual(nodir, compare_nodir)
        withdir = myvc.get_initial_structure_from_directory("withdir")
        compare_withdir = pymatgen.io.vaspio.Poscar.from_file("withdir/POSCAR").structure
        self.assertEqual(withdir, compare_withdir)

    def test_get_final_structure(self):
        myvc = VaspChecker(name = "structure")
        nodir = myvc.get_final_structure_from_directory()
        compare_nodir = pymatgen.io.vaspio.Poscar.from_file("structure/CONTCAR").structure
        self.assertEqual(nodir, compare_nodir)
        withdir = myvc.get_final_structure_from_directory("withdir")
        compare_withdir = pymatgen.io.vaspio.Poscar.from_file("withdir/CONTCAR").structure
        self.assertEqual(withdir, compare_withdir)
    def test_forward_final_structure(self):
        myvc = VaspChecker(name = "structure")
        myvc.forward_final_structure_file(os.path.join(testdir,"childdir"))
        fstrp = pymatgen.io.vaspio.Poscar.from_file("structure/CONTCAR").structure
        fstrc = pymatgen.io.vaspio.Poscar.from_file("childdir/POSCAR").structure
        self.assertEqual(fstrp, fstrc)

    def test_forward_initial_structure(self):
        myvc = VaspChecker(name = "structure")
        myvc.forward_initial_structure_file(os.path.join(testdir,"childdir"))
        istrp = pymatgen.io.vaspio.Poscar.from_file("structure/POSCAR").structure
        istrc = pymatgen.io.vaspio.Poscar.from_file("childdir/POSCAR").structure
        self.assertEqual(istrp, istrc)
    def test_forward_dynamical_matrix(self):
        myvc = VaspChecker(name = "dynamics")
        myvc.forward_dynamical_matrix_file(os.path.join(testdir,"childdir"))
        dp = myvc.read_my_dynamical_matrix_file("dynamics")
        dc = myvc.read_my_dynamical_matrix_file("childdir")
        self.assertEqual(dp,dc)

    def test_forward_displacement(self):
        myvc = VaspChecker(name="dynamics")
        myvc.forward_displacement_file(os.path.join(testdir,"childdir"))
        dp = myvc.read_my_displacement_file("dynamics")
        dc = myvc.read_my_displacement_file("childdir")
        self.assertEqual(dp,dc)


    def test_forward_energy_file(self):
        myvc = VaspChecker(name="energy")
        myvc.forward_energy_file(os.path.join(testdir, "childdir"))
        op = pymatgen.io.vaspio.Outcar("energy/OSZICAR")
        oc = pymatgen.io.vaspio.Outcar("childdir/OSZICAR")
        self.assertEqual(op.run_stats,oc.run_stats)
    def test_is_complete(self):
        vcc = VaspChecker(name="done")
        vcs = VaspChecker(name="started")
        self.assertFalse(vcs.is_complete())
        self.assertTrue(vcc.is_complete())

    def test_is_complete_additional_cases(self):
        kdict=dict()
        kdict['ibrion'] = "-1"
        kdict['nsw'] = "0"
        vcs = VaspChecker(name="done_static",program_keys=kdict)
        self.assertTrue(vcs.is_complete())
        vcs = VaspChecker(name="done_static_butnotconverged",program_keys=kdict)
        self.assertFalse(vcs.is_complete())
        vcs = VaspChecker(name="notdone_static_notconverged",program_keys=kdict)
        self.assertFalse(vcs.is_complete())
        kdict=dict()
        kdict['ibrion'] = "2"
        kdict['nsw'] = "191"
        vcr = VaspChecker(name="notdone_notconverged",program_keys=kdict)
        self.assertFalse(vcr.is_complete())
        vcr = VaspChecker(name="done_butnotconverged",program_keys=kdict)
        self.assertFalse(vcr.is_complete())
        kdict=dict()
        kdict['ibrion'] = "0"
        kdict['nsw'] = "191"
        vcr = VaspChecker(name="done_butnotconverged",program_keys=kdict)
        self.assertTrue(vcr.is_complete())
        vcr = VaspChecker(name="notdone_notconverged",program_keys=kdict)
        self.assertFalse(vcr.is_complete())
        kdict=dict()
        kdict['ibrion'] = "5"
        kdict['nsw'] = "191"
        vcr = VaspChecker(name="done_butnotconverged",program_keys=kdict)
        self.assertTrue(vcr.is_complete())
        vcr = VaspChecker(name="notdone_notconverged",program_keys=kdict)
        self.assertFalse(vcr.is_complete())

    def test_is_ready(self):
        vcnr1 = VaspChecker(name="notready1")
        vcnr2 = VaspChecker(name="notready2")
        vcnr3 = VaspChecker(name="notready3")
        vcnr4 = VaspChecker(name="notready4")
        vcnr5 = VaspChecker(name="notready5")
        vcr = VaspChecker(name="ready")
        self.assertFalse(vcnr1.is_ready_to_run())
        self.assertFalse(vcnr2.is_ready_to_run())
        self.assertFalse(vcnr3.is_ready_to_run())
        self.assertFalse(vcnr4.is_ready_to_run())
        self.assertFalse(vcnr5.is_ready_to_run())
        self.assertTrue(vcr.is_ready_to_run())

    def test_combine_dynamical_matrix(self):
        myvc = VaspChecker(name="dynamics_split")
        myvc.combine_dynamical_matrix_files()
        shutil.move(os.path.join(testdir, "dynamics_split/DYNMAT"),os.path.join(testdir,"childdir"))
        shutil.move(os.path.join(testdir, "dynamics_split/DYNMAT_combined"),os.path.join(testdir,"childdir"))
        dynmat_compare = myvc.read_my_dynamical_matrix_file(myvc.keywords['name'],"DYNMAT_compare")
        dynmat_combined = myvc.read_my_dynamical_matrix_file("childdir","DYNMAT_combined")
        self.assertEqual(dynmat_compare, dynmat_combined)

    def test_vasp_poscar_setup_has_poscar(self):
        compare_pos = pymatgen.io.vaspio.Poscar.from_file(os.path.join(testdir, "structure","POSCAR_perfect"))
        compare_pos.write_file(os.path.join(testdir,"childdir","POSCAR"))
        myvc = VaspChecker(name="childdir")
        mypos = myvc._vasp_poscar_setup()
        self.assertEqual(mypos.structure, compare_pos.structure)

    def test_vasp_poscar_setup_no_poscar(self):
        compare_pos = pymatgen.io.vaspio.Poscar.from_file(os.path.join(testdir, "structure","POSCAR_perfect"))
        myvc = VaspChecker(name="childdir",structure=compare_pos.structure)
        self.assertFalse(os.path.isfile(os.path.join(testdir, "childdir","POSCAR")))
        myvc._vasp_poscar_setup()
        mypos = pymatgen.io.vaspio.Poscar.from_file(os.path.join(testdir, "childdir","POSCAR"))
        self.assertEqual(mypos.structure, compare_pos.structure)

    def test_vasp_poscar_setup_mast_coordinates(self):
        kdict=dict()
        kdict['mast_coordinates'] = ["structure/POSCAR_coordinates"] #note that the input is a list of strings
        myvc = VaspChecker(name="childdir", program_keys=kdict)
        perf_pos = pymatgen.io.vaspio.Poscar.from_file(os.path.join(testdir,"structure","POSCAR_perfect"))
        perf_pos.write_file(os.path.join(testdir,"childdir","POSCAR"))
        coord_pos = pymatgen.io.vaspio.Poscar.from_file(os.path.join(testdir,"structure","POSCAR_coordinates"))
        grafted_pos = pymatgen.io.vaspio.Poscar.from_file(os.path.join(testdir,"structure","POSCAR_grafted"))
        mypos = myvc._vasp_poscar_setup()
        self.assertEqual(mypos.structure, grafted_pos.structure)

    def test_vasp_kpoints_setup_from_metafile(self):
        kdict=dict()
        mymeta = MASTFile("childdir/metadata.txt")
        mymeta.data.append("kpoints = 3x1x7 G 0.5 0.2 .1\n")
        mymeta.to_file("childdir/metadata.txt")
        mymeta2=MASTFile("childdir/metadata.txt")
        print mymeta2.data
        myvc = VaspChecker(name="childdir",program_keys=kdict)
        mykpt = myvc._vasp_kpoints_setup()
        kpt_compare = pymatgen.io.vaspio.Kpoints.from_file("files/KPOINTS_317G")
        self.assertEqual(kpt_compare.kpts[0][0],mykpt.kpts[0][0])
        self.assertEqual(kpt_compare.kpts[0][1],mykpt.kpts[0][1])
        self.assertEqual(kpt_compare.kpts[0][2],mykpt.kpts[0][2])
        self.assertEqual(kpt_compare.num_kpts, mykpt.num_kpts)
        self.assertEqual(kpt_compare.style, mykpt.style)
        #self.assertEqual(kpt_compare.kpts_shift, mykpt.kpts_shift)
        self.assertEqual((0.5,0.2,0.1),mykpt.kpts_shift)


    def test_vasp_kpoints_setup_from_keyword(self):
        kdict=dict()
        kdict['mast_kpoints'] = [3,3,3,"M"]
        myvc = VaspChecker(name="childdir",program_keys=kdict)
        mykpt=myvc._vasp_kpoints_setup()
        kpt_compare = pymatgen.io.vaspio.Kpoints.from_file("files/KPOINTS_333M")
        self.assertEqual(kpt_compare.kpts[0][0],mykpt.kpts[0][0])
        self.assertEqual(kpt_compare.kpts[0][1],mykpt.kpts[0][1])
        self.assertEqual(kpt_compare.kpts[0][2],mykpt.kpts[0][2])
        self.assertEqual(kpt_compare.num_kpts, mykpt.num_kpts)
        self.assertEqual(kpt_compare.style, mykpt.style)
        
        os.remove("childdir/KPOINTS")
        kdict['mast_kpoints'] = [1,2,5,"G"]
        myvc = VaspChecker(name="childdir",program_keys=kdict)
        mykpt=myvc._vasp_kpoints_setup()
        kpt_compare = pymatgen.io.vaspio.Kpoints.from_file("files/KPOINTS_125G")
        self.assertEqual(kpt_compare.kpts[0][0],mykpt.kpts[0][0])
        self.assertEqual(kpt_compare.kpts[0][1],mykpt.kpts[0][1])
        self.assertEqual(kpt_compare.kpts[0][2],mykpt.kpts[0][2])
        self.assertEqual(kpt_compare.num_kpts, mykpt.num_kpts)
        self.assertEqual(kpt_compare.style, mykpt.style)

        os.remove("childdir/KPOINTS")
        kdict['mast_kpoints'] = [2,2,2,"throw error"]
        myvc = VaspChecker(name="childdir",program_keys=kdict)
        self.assertRaises(MASTError, myvc._vasp_kpoints_setup)

    def test_vasp_potcar_setup(self):
        kdict=dict()
        kdict['mast_xc'] = "pbe"
        my_poscar = pymatgen.io.vaspio.Poscar.from_file("structure/POSCAR_Al")
        myvc = VaspChecker(name="childdir",program_keys=kdict)
        mypotcar=myvc._vasp_potcar_setup(my_poscar)
        mypotcar = pymatgen.io.vaspio.Potcar.from_file("childdir/POTCAR")
        potcar_compare = pymatgen.io.vaspio.Potcar.from_file("files/POTCAR_Al_PBE")
        self.assertEqual(potcar_compare[0],mypotcar[0])
        
        kdict['mast_xc'] = "pw91"
        kdict['mast_pp_setup']={'La':'La','Ni':'Ni_pv','O':'O_s'}
        my_poscar = pymatgen.io.vaspio.Poscar.from_file("structure/POSCAR_LNO")
        myvc = VaspChecker(name="childdir",program_keys=kdict)
        mypotcar=myvc._vasp_potcar_setup(my_poscar)
        mypotcar = pymatgen.io.vaspio.Potcar.from_file("childdir/POTCAR")
        potcar_compare = pymatgen.io.vaspio.Potcar.from_file("files/POTCAR_LNO_PW91")
        self.assertEqual(potcar_compare[0],mypotcar[0])

    def test_vasp_incar_setup(self):
        my_structure = pymatgen.io.vaspio.Poscar.from_file("structure/POSCAR_LNO").structure
        kdict=dict()
        kdict['mast_xc'] = "pw91"
        kdict['mast_pp_setup']={'La':'La','Ni':'Ni_pv','O':'O_s'}
        kdict['IBRION'] = 3
        myvc = VaspChecker(name="childdir",program_keys=kdict,structure=my_structure)
        mypos = myvc._vasp_poscar_setup()
        mypot = myvc._vasp_potcar_setup(mypos)
        myvc._vasp_incar_setup(mypot, mypos)
        myincar = pymatgen.io.vaspio.Incar.from_file(os.path.join(testdir,"childdir","INCAR"))
        incar_compare = pymatgen.io.vaspio.Incar.from_file(os.path.join(testdir,"files","INCAR_notags"))
        self.assertEqual(myincar, incar_compare)
        #
        kdict['ENCUT'] = 100
        kdict['mast_multiplyencut'] = 1.1
        myvc = VaspChecker(name="childdir",program_keys=kdict,structure=my_structure)
        myvc._vasp_incar_setup(mypot, mypos)
        myincar = pymatgen.io.vaspio.Incar.from_file(os.path.join(testdir,"childdir","INCAR"))
        incar_compare = pymatgen.io.vaspio.Incar.from_file(os.path.join(testdir,"files","INCAR_encut"))
        self.assertEqual(myincar, incar_compare)
        #
        kdict.pop("ENCUT")
        myvc = VaspChecker(name="childdir",program_keys=kdict,structure=my_structure)
        myvc._vasp_incar_setup(mypot, mypos)
        myincar = pymatgen.io.vaspio.Incar.from_file(os.path.join(testdir,"childdir","INCAR"))
        incar_compare = pymatgen.io.vaspio.Incar.from_file(os.path.join(testdir,"files","INCAR_multiply_110"))
        self.assertEqual(myincar, incar_compare)
        #
        kdict.pop("mast_multiplyencut")
        kdict["mast_charge"] = -2
        myvc = VaspChecker(name="childdir",program_keys=kdict,structure=my_structure)
        myvc._vasp_incar_setup(mypot, mypos)
        myincar = pymatgen.io.vaspio.Incar.from_file(os.path.join(testdir,"childdir","INCAR"))
        incar_compare = pymatgen.io.vaspio.Incar.from_file(os.path.join(testdir,"files","INCAR_charge_neg"))
        self.assertEqual(myincar, incar_compare)
        #
        kdict["mast_charge"] = 2
        myvc = VaspChecker(name="childdir",program_keys=kdict,structure=my_structure)
        myvc._vasp_incar_setup(mypot, mypos)
        myincar = pymatgen.io.vaspio.Incar.from_file(os.path.join(testdir,"childdir","INCAR"))
        incar_compare = pymatgen.io.vaspio.Incar.from_file(os.path.join(testdir,"files","INCAR_charge_pos"))
        self.assertEqual(myincar, incar_compare)
        #
        kdict.pop("mast_charge")
        kdict["mast_setmagmom"]="3 5 2"
        myvc = VaspChecker(name="childdir",program_keys=kdict,structure=my_structure)
        myvc._vasp_incar_setup(mypot, mypos)
        myincar = pymatgen.io.vaspio.Incar.from_file(os.path.join(testdir,"childdir","INCAR"))
        incar_compare = pymatgen.io.vaspio.Incar.from_file(os.path.join(testdir,"files","INCAR_setmagmom"))
        self.assertEqual(myincar, incar_compare)
        #
        kdict["mast_setmagmom"]="-1 -2 3 1 2 3 1 2 7 8 7 8 7 8 7 8 5 4 3 2 1 1 2 3 4 5 -5 -4 -3 -2 -1 1 2 3 4 5 4 4 4 -4"
        myvc = VaspChecker(name="childdir",program_keys=kdict,structure=my_structure)
        myvc._vasp_incar_setup(mypot, mypos)
        myincar = pymatgen.io.vaspio.Incar.from_file(os.path.join(testdir,"childdir","INCAR"))
        incar_compare = pymatgen.io.vaspio.Incar.from_file(os.path.join(testdir,"files","INCAR_setmagmom_indiv"))
        self.assertEqual(myincar, incar_compare)

    def test_set_up_program_input(self):
        my_structure = pymatgen.io.vaspio.Poscar.from_file("structure/POSCAR_Al").structure
        kdict=dict()
        kdict['IBRION']="2"
        kdict['ISIF']="3"
        kdict['ISMEAR']="1"
        kdict['LCHARG']="FALSE"
        kdict['LWAVE']="FALSE"
        kdict['NSW']="191"
        kdict['PREC']="Accurate"
        kdict['SIGMA']="0.2"
        kdict['mast_kpoints']=[1,1,1,"G"]
        kdict['mast_xc']="pw91"
        myvc = VaspChecker(name="childdir",program_keys=kdict,structure=my_structure)
        myvc.set_up_program_input()
        myincar = pymatgen.io.vaspio.Incar.from_file("childdir/INCAR")
        myposcar = pymatgen.io.vaspio.Poscar.from_file("childdir/POSCAR")
        mypotcar = pymatgen.io.vaspio.Potcar.from_file("childdir/POTCAR")
        mykpts = pymatgen.io.vaspio.Kpoints.from_file("childdir/KPOINTS")
        compare_incar = pymatgen.io.vaspio.Incar.from_file("ready/INCAR")
        compare_poscar = pymatgen.io.vaspio.Poscar.from_file("ready/POSCAR")
        compare_potcar = pymatgen.io.vaspio.Potcar.from_file("ready/POTCAR")
        compare_kpoints = pymatgen.io.vaspio.Kpoints.from_file("ready/KPOINTS")
        self.assertEqual(myincar, compare_incar)
        self.assertEqual(myposcar.structure, compare_poscar.structure)
        self.assertEqual(mykpts.kpts[0][0],compare_kpoints.kpts[0][0])
        self.assertEqual(mykpts.kpts[0][1],compare_kpoints.kpts[0][1])
        self.assertEqual(mykpts.kpts[0][2],compare_kpoints.kpts[0][2])
        self.assertEqual(mykpts.num_kpts, compare_kpoints.num_kpts)
        self.assertEqual(mykpts.style, compare_kpoints.style)


        self.assertEqual(mypotcar, compare_potcar)

    #def test_forward_extra_restart_files(self):
    #    myvc = VaspChecker(name="files")
    #    myvc.forward_extra_restart_files("childdir")
    #    myfiles=dirutil.walkfiles("childdir")
    #    self.assertTrue("childdir/WAVECAR" in myfiles)
    #    self.assertTrue("childdir/CHGCAR" in myfiles)

    def test_softlink_charge_density_file(self):
        import subprocess
        myvc = VaspChecker(name="files")
        myvc.softlink_charge_density_file("childdir")
        myfiles=dirutil.walkfiles("childdir")
        print myfiles
        #self.assertTrue("childdir/CHGCAR" in myfiles)
        listme=subprocess.Popen("ls -l childdir",stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
        listres=listme.communicate()[0]
        listme.wait()
        print listres
        self.assertTrue("files/CHGCAR" in listres)
    def test_softlink_wavefunction_file(self):
        import subprocess
        myvc = VaspChecker(name="files")
        myvc.softlink_wavefunction_file("childdir")
        myfiles=dirutil.walkfiles("childdir")
        print myfiles
        #self.assertTrue("childdir/CHGCAR" in myfiles)
        listme=subprocess.Popen("ls -l childdir",stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
        listres=listme.communicate()[0]
        listme.wait()
        print listres
        self.assertTrue("files/WAVECAR" in listres)

    def test_forward_charge_density_file(self):
        myvc = VaspChecker(name="files")
        myvc.forward_charge_density_file("childdir")
        self.assertTrue(os.path.isfile("childdir/CHGCAR"))
    def test_forward_wavefunction_file(self):
        myvc = VaspChecker(name="files")
        myvc.forward_wavefunction_file("childdir")
        self.assertTrue(os.path.isfile("childdir/WAVECAR"))
    def test_add_selective_dynamics(self):
        mystruc = pymatgen.io.vaspio.Poscar.from_file(os.path.join(testdir,"structure","POSCAR_Al")).structure
        myvc = VaspChecker(name="childdir",structure=mystruc)
        myvc._vasp_poscar_setup()
        sdarr = np.zeros([4,3],bool)
        sdarr[0][0]=True
        sdarr[1][1]=True
        sdarr[2][2]=True
        sdarr[3][0]=True
        sdarr[3][1]=True
        myvc.add_selective_dynamics_to_structure_file(sdarr)
        after_pos = pymatgen.io.vaspio.Poscar.from_file(os.path.join(testdir,"childdir","POSCAR"))
        compare_pos = pymatgen.io.vaspio.Poscar.from_file(os.path.join(testdir,"structure","POSCAR_Al_sd"))
        self.assertEqual(after_pos.structure, compare_pos.structure)
        self.assertEqual(after_pos.selective_dynamics, compare_pos.selective_dynamics)

    def test_get_non_mast_keywords(self):
        kdict=dict()
        kdict['mast_nokeyword']="hello"
        kdict['mast_charge']="1"
        kdict['IBRION']="3"
        kdict['skip_keyword']="skip"
        myvc = VaspChecker(name="childdir",program_keys=kdict)
        keydict=myvc._vasp_incar_get_non_mast_keywords()
        self.assertEqual(keydict,{"IBRION":"3"})

    def test_get_energy(self):
        myvc = VaspChecker(name="done")
        myenergy=myvc.get_energy()
        self.assertAlmostEqual(myenergy, float(-.83312666E+01),places=7)

    def test_get_max_enmax_from_potcar(self):
        mypotcar = pymatgen.io.vaspio.Potcar.from_file(os.path.join(testdir,"files","POTCAR_LNO_PW91"))
        myvc = VaspChecker(name="childdir")
        maxenmax = myvc._get_max_enmax_from_potcar(mypotcar)
        self.assertEqual(maxenmax, 367.921)
        mypotcar = pymatgen.io.vaspio.Potcar.from_file(os.path.join(testdir,"files","POTCAR_LTO_PW91"))
        maxenmax = myvc._get_max_enmax_from_potcar(mypotcar)
        self.assertEqual(maxenmax, 400)

    def test_read_my_dynamical_matrix_file(self):
        myvc = VaspChecker(name="dynamics")
        mydm=myvc.read_my_dynamical_matrix_file()
        self.assertEqual(mydm['numspec'],1)
        self.assertEqual(mydm['numatoms'],4)
        self.assertEqual(mydm['numdisp'],3)
        self.assertEqual(mydm['massline']," 55.847\n")
        self.assertEqual(mydm['atoms'][1][1]['displine'],"0.0100 0.0000 0.0000")
        self.assertEqual(mydm['atoms'][1][3]['displine'],"0.0000 0.0000 0.0100")
        self.assertEqual(mydm['atoms'][1][1]['dynmat'][0]," -0.101036   0.000000   0.000000\n")
        self.assertEqual(mydm['atoms'][1][3]['dynmat'][2],"  0.000000   0.000000   0.000000\n")

    def test_write_my_dynamical_matrix_file(self):
        myvc = VaspChecker(name="childdir")
        mydm=myvc.read_my_dynamical_matrix_file("dynamics")
        myvc.write_my_dynamical_matrix_file(mydm)
        mydm2 = myvc.read_my_dynamical_matrix_file()
        self.assertEqual(mydm,mydm2)
    def test_write_my_dynamical_matrix_file_without_disp_mass(self):
        myvc = VaspChecker(name="childdir")
        mydm=myvc.read_my_dynamical_matrix_file("dynamics")
        myvc.write_my_dynmat_without_disp_or_mass(mydm)
        myread = MAST.utility.MASTFile("dynamics/DYNMAT_for_PHON")
        myread2 = MAST.utility.MASTFile("childdir/DYNMAT")
        self.assertEqual(myread2.data,myread.data)

    def test_read_my_displacement_file(self):
        myvc = VaspChecker(name="dynamics")
        mydisp = myvc.read_my_displacement_file(myvc.keywords['name'],"XDATCAR_vasp522")
        self.assertEqual(mydisp['numatoms'],4)
        self.assertEqual(mydisp['configs'][6][0],"   0.00000000  0.00000000  0.00286262\n")
        mydisp = myvc.read_my_displacement_file(myvc.keywords['name'],"XDATCAR_vasp5211")
        self.assertEqual(mydisp['numatoms'],4)
        self.assertEqual(mydisp['configs'][2][1],"   0.50244722  0.50000000  0.00000000\n")

    def test_write_my_displacement_file(self):
        myvc = VaspChecker(name="childdir")
        mydisp = myvc.read_my_displacement_file("dynamics","XDATCAR_vasp5211")
        myvc.write_my_displacement_file(mydisp)
        mydisp2 = myvc.read_my_displacement_file()
        self.assertEqual(mydisp,mydisp2)
        mydisp = myvc.read_my_displacement_file("dynamics","XDATCAR_vasp522")
        myvc.write_my_displacement_file(mydisp)
        mydisp2 = myvc.read_my_displacement_file()
        self.assertEqual(mydisp,mydisp2)

    def test_combine_displacement_files(self):
        myvc = VaspChecker(name="dynamics_split")
        myvc.combine_displacement_files()
        shutil.move(os.path.join(testdir, "dynamics_split/XDATCAR"),os.path.join(testdir,"childdir"))
        shutil.move(os.path.join(testdir, "dynamics_split/XDATCAR_combined"),os.path.join(testdir,"childdir"))
        disp_compare = myvc.read_my_displacement_file(myvc.keywords['name'],"XDATCAR_compare")
        disp_combined = myvc.read_my_displacement_file("childdir","XDATCAR_combined")
        print "COMPARE"
        for key,value in disp_compare.iteritems():
            self.assertEqual(value, disp_combined[key])
    def test_get_total_electrons(self):
        mypos = pymatgen.io.vaspio.Poscar.from_file("structure/POSCAR_LTO")
        mypot = pymatgen.io.vaspio.Potcar.from_file("files/POTCAR_LTO_PW91")
        myvc = VaspChecker()
        numelec = myvc.get_total_electrons(mypos, mypot)
        self.assertEqual(numelec,328)
    def test_get_valence_list(self):
        mypot = pymatgen.io.vaspio.Potcar.from_file("files/POTCAR_LTO_PW91")
        myvc = VaspChecker()
        vlist = myvc.get_valence_list(mypot)
        self.assertEqual(vlist,[11,12,6])
    def test_get_energy_from_energy_file(self):
        myvc = VaspChecker(name="done")
        myenergy=myvc.get_energy_from_energy_file()
        self.assertAlmostEqual(myenergy,float(-.83312666E+01),places=7)
    def test_is_started(self):
        myvc = VaspChecker(name="notready1")
        self.assertFalse(myvc.is_started())
        myvc = VaspChecker(name="ready")
        self.assertFalse(myvc.is_started())
        myvc = VaspChecker(name="started")
        self.assertTrue(myvc.is_started())
        myvc = VaspChecker(name="done")
        self.assertTrue(myvc.is_started())
    def test_write_final_structure_file(self):
        myvc = VaspChecker(name="childdir")
        finalstruc = pymatgen.io.vaspio.Poscar.from_file("structure/CONTCAR").structure
        myvc.write_final_structure_file(finalstruc)
        mystruc = pymatgen.io.vaspio.Poscar.from_file("childdir/CONTCAR").structure
        self.assertEqual(mystruc,finalstruc)
    def test_has_starting_structure_file(self):
        myvc = VaspChecker(name="notready3")
        self.assertFalse(myvc.has_starting_structure_file())
        myvc = VaspChecker(name="ready")
        self.assertTrue(myvc.has_starting_structure_file())
        myvc = VaspChecker(name="notready1")
        self.assertTrue(myvc.has_starting_structure_file())
        myvc = VaspChecker(name="notready2")
        self.assertTrue(myvc.has_starting_structure_file())
        myvc = VaspChecker(name="notready4")
        self.assertTrue(myvc.has_starting_structure_file())
        myvc = VaspChecker(name="notready5")
        self.assertTrue(myvc.has_starting_structure_file())
    def test_has_ending_structure_file(self):
        myvc = VaspChecker(name="ready")
        self.assertFalse(myvc.has_ending_structure_file())
        myvc = VaspChecker(name="done")
        self.assertTrue(myvc.has_ending_structure_file())
    def test_scale_mesh(self):
        myvc = VaspChecker()
        mystr = pymatgen.io.vaspio.Poscar.from_file("ready/POSCAR").structure
        myk=myvc.scale_mesh(mystr, 2000)
        self.assertEqual(myvc.keywords['program_keys']['mast_kpoints'],[8,8,8,'M'])



import os
import random
from ase import Atom, Atoms
from ase.optimize import BFGS
from MAST.structopt.tools.lammps import LAMMPS
from MAST.structopt.tools.setup_calculator import setup_calculator

def ase_minimization(indiv, Optimizer):
    """Function to use built in ASE minimizers to minimize atomic positions in structure.
    Input:
        indiv = Individual class object to be optimized
        Optimizer = Optimizer class object with needed parameters
    Output:
        indiv = Optimized Individual class object.
    """
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    cwd1=os.getcwd()
    olammpsmin = Optimizer.lammps_min
    if Optimizer.lammps_min:
        Optimizer.lammps_min = None
    calc2 = setup_calculator(Optimizer)
#     if 'mass' in indiv[0].get_calculator():
#         mass2 = ['1 '+ str(Optimizer.atomlist[0][2])]
#         if len(Optimizer.atomlist) > 1:
#             for i in range(len(Optimizer.atomlist)-1):
#                 mass2.append(str(i+2) + ' ' + str(Optimizer.atomlist[i+1][2]))
#         calc2=LAMMPS(parameters={ 'pair_style' : Optimizer.pair_style, 'pair_coeff' : Optimizer.pair_coeff , 'mass' : mass2 },files=[ Optimizer.pot_file ])
#     else:
#         calc2=LAMMPS(parameters={ 'pair_style' : Optimizer.pair_style, 'pair_coeff' : Optimizer.pair_coeff},files=[ Optimizer.pot_file ])
    if Optimizer.structure==Defect:
        nat=indiv[0].get_number_of_atoms
        sol=Atoms()
        sol.extend(indiv[0])
        sol.extend(indiv.bulko)
        sol.set_calculator(calc2)
        sol.set_cell(indiv.bulko.get_cell())
        sol.set_pbc(True)
        dyn=BFGS(sol)
        dyn.run(fmax=0.001, steps=2500)
        positions=sol[0:nat].get_positions()
        indiv[0].set_positions(positions)
    else:
        atomsdup=indiv[0].copy()
        atomsdup.set_calculator(calc2)
        dyn=BFGS(indiv[0])
        dyn.run(fmax=0.001, steps=2500)
        positions=atomsdup.get_positions()
        indiv[0].set_positions(positions)
    os.chdir(cwd1)
    calc2.clean()
    Optimizer.lammps_min = olammpsmin
    Optimizer.output.write('ASE Minimization mutation performed on individual = '+repr(indiv.index)+'\n')
    muttype='ASEM'
    if indiv.energy==0:
        indiv.history_index=indiv.history_index+'m'+muttype
    else:
        indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv
		
		


import random
import numpy
from ase import Atom, Atoms
from MAST.structopt.inp_out import write_xyz

def rotct(ind1, ind2, Optimizer):
    """Rotate atoms cut and splice
    Rotates atoms randomly around center of mass and cuts with xy plane
    Maintains number of atoms
    Maintains concentration of atoms
    Returns individuals to standard positions at end (un-rotates)
    """
    if 'CX' in Optimizer.debug:
        debug = True
    else:
        debug = False
    Optimizer.output.write('Rotate Cut/Splice Cx between individual '+repr(ind1.index)+' and individual '+repr(ind2.index)+'\n')
    
    #Perserve starting conditions of individual
    indi1 = ind1[0].copy()
    indi2 = ind2[0].copy()
    
    #Translate individuals so COM is at (0,0,0)
    com1 = indi1.get_center_of_mass()
    indi1.translate(-1*com1)
    com2 = indi2.get_center_of_mass()
    indi2.translate(-1*com2)
    #Select random axis and random angle and rotate individuals
    n=0
    while n<10:
        rax = random.choice(['x', '-x','y','-y','z','-z'])
        rang = random.random()*90
        indi1.rotate(rax,a=rang,center='COM',rotate_cell=False)
        #Search for atoms in individual 1 that are above the xy plane
        group1 = Atoms(cell=ind1[0].get_cell(),pbc=ind1[0].get_pbc())
        indices1=[]
        for one in indi1:
            if one.position[2] >= 0:
                group1.append(one)
                indices1.append(one.index)
        if len(group1) > 2 and len(group1) < len(indi1):
            break
        else:
            indi1.rotate(rax,a=-1*rang,center='COM', rotate_cell=False)
        n+=1
    indi2.rotate(rax,a=rang,center='COM', rotate_cell=False)
    if debug: 
        print 'Group1 size = ', len(group1)
        print 'Position = ', [0,0,0]
        print 'Angle = ', rang
        print 'Axis = ', rax
        print 'Number of tries = ', n+1
    #Apply concentration forcing if needed
    group2 = Atoms(cell=ind2[0].get_cell(),pbc=ind2[0].get_pbc())
    indices2 = []
    dellist = []
    if Optimizer.forcing=='Concentration':
        symlist = list(set(group1.get_chemical_symbols()))
        seplist = [[atm for atm in indi2 if atm.symbol == sym] for sym, count in symlist]
        for one in group1:
            sym1=one.symbol
            listpos=[i for i,s in enumerate(symlist) if s==sym1][0]
            if len(seplist[listpos]) > 0:
                dist=[z for x,y,z in [one.position for one in seplist[listpos]]]
                pos = [i for i,value in enumerate(dist) if value==max(dist)][0]
                group2.append(seplist[listpos][pos])
                indices2.append(seplist[listpos][pos].index)
                del seplist[listpos][pos]
            else:
                dellist.append(one.index)
        if len(dellist) != 0:
            dellist.sort(reverse=True)
            for one in dellist:
                del group1[one]
                del indices1[one]
        #indices2.append(pt2.index)
    else:
        for one in indi2:
            if one.position[2] >= 0:
                group2.append(one)
                indices2.append(one.index)
        while len(group2) < len(group1)-len(dellist):
            #Too many atoms in group 1
            dellist.append(random.choice(group1).index)
        if len(dellist) != 0:
            dellist.sort(reverse=True)
            for one in dellist:
                del group1[one]
                del indices1[one]
        dellist = []
        while len(group1) < len(group2)-len(dellist):
            #Too many atoms in group 2
            dellist.append(random.choice(group2).index)
        if len(dellist) != 0:
            dellist.sort(reverse=True)
            for one in dellist:
                del group2[one]
                del indices2[one]
    
    other1 = Atoms()
    other2 = Atoms()
    other2 = Atoms(cell=ind2[0].get_cell(),pbc=ind2[0].get_pbc())
    for one in indi2:
        if one.index not in indices2:
            other2.append(one)
    other1 = Atoms(cell=ind1[0].get_cell(),pbc=ind1[0].get_pbc())
    for one in indi1:
        if one.index not in indices1:
            other1.append(one)
    

    indi1 = group2.copy()
    indi1.extend(other1)
    indi2 = group1.copy()
    indi2.extend(other2)
    
    #DEBUG: Write crossover to file
    if debug: 
        write_xyz(Optimizer.debugfile, group1,'group1')
        write_xyz(Optimizer.debugfile, other1,'other1')
        write_xyz(Optimizer.debugfile, group2,'group2')
        write_xyz(Optimizer.debugfile, other2,'other2')
        print 'Length of group1 = ',len(group1),'Length of group2',len(group2)
    
    
    #DEBUG: Check structure of atoms exchanged
    for sym,c,m,u in Optimizer.atomlist:
        nc=len([atm for atm in indi1 if atm.symbol==sym])
        Optimizer.output.write('CX ROTCT: Individual 1 contains '+repr(nc)+' '+repr(sym)+' atoms\n')
        nc=len([atm for atm in indi2 if atm.symbol==sym])
        Optimizer.output.write('CX ROTCT: Individual 2 contains '+repr(nc)+' '+repr(sym)+' atoms\n')

    #Need to have at least one atom of each structure in atomlist to prevent Lammps for erroring
    if Optimizer.forcing !='Concentration':
        for i in range(len(Optimizer.atomlist)):
            atms1=[inds for inds in indi1 if inds.symbol==Optimizer.atomlist[i][0]]
            atms2=[inds for inds in indi2 if inds.symbol==Optimizer.atomlist[i][0]]
            if len(atms1)==0:
                if len(atms2)==0:
                    indi1[random.randint(0,len(indi1)-1)].symbol==Optimizer.atomlist[i][0]
                    indi2[random.randint(0,len(indi2)-1)].symbol==Optimizer.atomlist[i][0]
                else:
                    indi1.append(atms2[random.randint(0,len(atms2)-1)])
                    indi1.pop(random.randint(0,len(indi1)-2))
            else:
                if len(atms2)==0:
                    indi2.append(atms1[random.randint(0,len(atms1)-1)])
                    indi2.pop(random.randint(0,len(indi2)-2))	
    indi1.rotate(rax,a=-1*rang,center='COM', rotate_cell=False)
    indi2.rotate(rax,a=-1*rang,center='COM', rotate_cell=False)
    indi1.translate(com1)
    indi2.translate(com2)

    if Optimizer.structure != 'Defect':
        cm = indi1.get_center_of_mass()
        cell = numpy.maximum.reduce(indi1.get_cell())
        cop = [cell[0]/float(2), cell[1]/float(2), cell[2]/float(2)]
        indi1.translate(-1*cm)
        indi1.translate(cop)
        cm = indi2.get_center_of_mass()
        cell = numpy.maximum.reduce(indi2.get_cell())
        cop = [cell[0]/float(2), cell[1]/float(2), cell[2]/float(2)]
        indi2.translate(-1*cm)
        indi2.translate(cop)

    ind1[0]=indi1
    ind2[0]=indi2
    
    #Check structure and number of atoms in crystal
    if Optimizer.structure=='Defect':
        solid1=Atoms()
        solid1.extend(ind1[0])
        solid1.extend(ind1.bulki)
        solid2=Atoms()
        solid2.extend(ind1[0])
        solid2.extend(ind2.bulki)
        for sym,c,m,u in Optimizer.atomlist:
                nc=len([atm for atm in solid1 if atm.symbol==sym])
                Optimizer.output.write('CX ROTCT: CIBS 1 configuration contains '+repr(nc)+' '+repr(sym)+' atoms\n')
                nc=len([atm for atm in solid2 if atm.symbol==sym])
                Optimizer.output.write('CX ROTCT: CIBS 2 configuration contains '+repr(nc)+' '+repr(sym)+' atoms\n')
    return ind1, ind2



import os
from ase import Atom, Atoms
from MAST.structopt.io.write_xyz import write_xyz
from MAST.structopt.tools.setup_calculator import setup_calculator
from MAST.structopt.tools.find_defects import find_defects
from MAST.structopt.tools.check_cell_type import check_cell_type
from MAST.structopt.fingerprinting import get_fingerprint
from MAST.structopt.tools.lammps import LAMMPS
from ase.calculators.vasp import Vasp
from ase.optimize import BFGS
from ase.units import GPa
from ase.calculators.neighborlist import NeighborList
import numpy
try:
    from mpi4py import MPI
except ImportError:
    pass
import pdb

def eval_energy(input):
    """Function to evaluate energy of an individual
    Inputs:
        input = [Optimizer class object with parameters, Individual class structure to be evaluated]
    Outputs:
        energy, bul, individ, signal
        energy = energy of Individual evaluated
        bul = bulk structure of Individual if simulation structure is Defect
        individ = Individual class structure evaluated
        signal = string of information about evaluation
    """
    if input[0]==None:
        energy=0
        bul=0
        individ=0
        rank = MPI.COMM_WORLD.Get_rank()
        signal='Evaluated none individual on '+repr(rank)+'\n'
    else:
        [Optimizer, individ]=input
    if Optimizer.calc_method=='MAST':
        energy = individ.energy
        bul = individ.energy
        signal = 'Recieved MAST structure\n'
    else:
        if Optimizer.parallel: rank = MPI.COMM_WORLD.Get_rank()
        if not Optimizer.genealogy:
            STR='----Individual ' + str(individ.index)+ ' Optimization----\n'
        else:
            STR='----Individual ' + str(individ.history_index)+ ' Optimization----\n'
        indiv=individ[0]
        if 'EE' in Optimizer.debug:
            debug = True
        else:
            debug = False
        if debug: 
            write_xyz(Optimizer.debugfile,indiv,'Recieved by eval_energy')
            Optimizer.debugfile.flush()
        if Optimizer.structure=='Defect':
            indi=indiv.copy()
            if Optimizer.alloy==True:
                bulk=individ.bulki
            else:
                bulk=individ.bulko
            nat=indi.get_number_of_atoms()
            csize=bulk.get_cell()                                                                                                         
            totalsol=Atoms(cell=csize, pbc=True)
            totalsol.extend(indi)
            totalsol.extend(bulk)
            for sym,c,m,u in Optimizer.atomlist:
                nc=len([atm for atm in totalsol if atm.symbol==sym])
                STR+='Defect configuration contains '+repr(nc)+' '+repr(sym)+' atoms\n'
    
        elif Optimizer.structure=='Surface':
            totalsol=Atoms()
            totalsol.extend(indiv)
            nat=indiv.get_number_of_atoms()
            totalsol.extend(individ.bulki)
            for sym,c,m,u in Optimizer.atomlist:
                nc=len([atm for atm in totalsol if atm.symbol==sym])
                STR+='Surface-Bulk configuration contains '+repr(nc)+' '+repr(sym)+' atoms\n'
            cell=numpy.maximum.reduce(indiv.get_cell())
            totalsol.set_cell([cell[0],cell[1],500])
            totalsol.set_pbc([True,True,False])
    
        if Optimizer.constrain_position:
            ts = totalsol.copy()
            indc,indb,vacant,swap,stro = find_defects(ts,Optimizer.solidbulk,0)
            sbulk = Optimizer.solidbulk.copy()
            bcom = sbulk.get_center_of_mass()
            #totalsol.translate(-bulkcom)
            #indc.translate(-bulkcom)
            #totalsol.append(Atom(position=[0,0,0]))
    # 			for one in indc:
    # 				index = [atm.index for atm in totalsol if atm.position[0]==one.position[0] and atm.position[1]==one.position[1] and atm.position[2]==one.position[2]][0]
    # 				if totalsol.get_distance(-1,index) > Optimizer.sf:
    # 					r = random.random()
    # 					totalsol.set_distance(-1,index,Optimizer.sf*r,fix=0)
    # 			totalsol.pop()
    # 			totalsol.translate(bulkcom)
            com = indc.get_center_of_mass()
            dist = (sum((bcom[i] - com[i])**2 for i in range(3)))**0.5
            if dist > Optimizer.sf:
                STR+='Shifting structure to within region\n'
                r = random.random()*Optimizer.sf
                comv = numpy.linalg.norm(com)
                ncom = [one*r/comv for one in com]
                trans = [ncom[i]-com[i] for i in range(3)]
                indices = []
                for one in indc:
                    id = [atm.index for atm in totalsol if atm.position[0]==one.position[0] and atm.position[1]==one.position[1] and atm.position[2]==one.position[2]][0]
                    totalsol[id].position += trans
    
        # Check for atoms that are too close
        min_len=0.7
        #pdb.set_trace()
        if not Optimizer.fixed_region:
            if Optimizer.structure=='Defect' or Optimizer.structure=='Surface':
                cutoffs=[2.0 for one in totalsol]
                nl=NeighborList(cutoffs,bothways=True,self_interaction=False)
                nl.update(totalsol)
                for one in totalsol[0:nat]:
                    nbatoms=Atoms()
                    nbatoms.append(one)
                    indices, offsets=nl.get_neighbors(one.index)
                    for index, d in zip(indices,offsets):
                        index = int(index)
                        sym=totalsol[index].symbol
                        pos=totalsol[index].position + numpy.dot(d,totalsol.get_cell())
                        at=Atom(symbol=sym,position=pos)
                        nbatoms.append(at)
                    while True:
                        dflag=False
                        for i in range(1,len(nbatoms)):
                            d=nbatoms.get_distance(0,i)
                            if d < min_len:
                                nbatoms.set_distance(0,i,min_len+.01,fix=0.5)
                                STR+='--- WARNING: Atoms too close (<0.7A) - Implement Move ---\n'
                                dflag=True
                        if dflag==False:
                            break
                    for i in range(len(indices)):
                        totalsol[indices[i]].position=nbatoms[i+1].position
                    totalsol[one.index].position=nbatoms[0].position
                    nl.update(totalsol)
                if debug:
                    write_xyz(Optimizer.debugfile,totalsol,'After minlength check')
                    Optimizer.debugfile.flush()
            else:
                for i in range(len(indiv)):
                    for j in range(len(indiv)):
                        if i != j:
                            d=indiv.get_distance(i,j)
                            if d < min_len:
                                indiv.set_distance(i,j,min_len,fix=0.5)
                                STR+='--- WARNING: Atoms too close (<0.7A) - Implement Move ---\n'
                if debug:
                    write_xyz(Optimizer.debugfile,indiv,'After minlength check')
                    Optimizer.debugfile.flush()
    
        # Set calculator to use to get forces/energies
        if Optimizer.parallel:
            calc = setup_calculator(Optimizer)
            if Optimizer.fixed_region:
                pms=copy.deepcopy(calc.parameters)
                try:
                    pms['mass'][len(pms['mass'])-1] += '\ngroup RO id >= '+repr(nat)+'\nfix freeze RO setforce 0.0 0.0 0.0\n'
                except KeyError:
                    pms['pair_coeff'][0] += '\ngroup RO id >= '+repr(nat)+'\nfix freeze RO setforce 0.0 0.0 0.0\n'
                calc = LAMMPS(parameters=pms, files=calc.files, keep_tmp_files=calc.keep_tmp_files, tmp_dir=calc.tmp_dir)
                lmin = copy.copy(Optimizer.lammps_min)
                Optimizer.lammps_min = None
                Optimizer.static_calc = setup_calculator(Optimizer)
                Optimizer.lammps_min = lmin
        else:
            calc=Optimizer.calc
        if Optimizer.structure=='Defect' or Optimizer.structure=='Surface':
            totalsol.set_calculator(calc)
            totalsol.set_pbc(True)
        else:
            indiv.set_calculator(calc)
            indiv.set_pbc(True)	#Current bug in ASE optimizer-Lammps prevents pbc=false 
            if Optimizer.structure=='Cluster':
                indiv.set_cell([500,500,500])
                indiv.translate([250,250,250])
    
        cwd=os.getcwd()
        # Perform Energy Minimization
        if not Optimizer.parallel:
            Optimizer.output.flush()
        if Optimizer.ase_min == True:
            try:
                if Optimizer.structure=='Defect' or Optimizer.structure=='Surface':
                    dyn=BFGS(totalsol)
                else:
                    dyn=BFGS(indiv)
                dyn.run(fmax=Optimizer.ase_min_fmax, steps=Optimizer.ase_min_maxsteps)
            except OverflowError:
                STR+='--- Error: Infinite Energy Calculated - Implement Random ---\n'
                box=Atoms()
                indiv=gen_pop_box(Optimizer.natoms, Optimizer.atomlist, Optimizer.size)
                indiv.set_calculator(calc)
                dyn=BFGS(indiv)
                dyn.run(fmax=fmax, steps=steps)
            except numpy.linalg.linalg.LinAlgError:
                STR+='--- Error: Singular Matrix - Implement Random ---\n'
                indiv=gen_pop_box(Optimizer.natoms, Optimizer.atomlist, Optimizer.size)
                indiv.set_calculator(calc)
                dyn=BFGS(indiv)
                dyn.run(fmax=fmax, steps=steps)
            # Get Energy of Minimized Structure
            if Optimizer.structure=='Defect' or Optimizer.structure=='Surface':
                en=totalsol.get_potential_energy()
                #force=numpy.maximum.reduce(abs(totalsol.get_forces()))
                if Optimizer.fitness_scheme == 'enthalpyfit':
                    pressure=totalsol.get_isotropic_pressure(totalsol.get_stress())
                    cell_max=numpy.maximum.reduce(totalsol.get_positions())
                    cell_min=numpy.minimum.reduce(totalsol.get_positions())
                    cell=cell_max-cell_min
                    volume=cell[0]*cell[1]*cell[2]
                else:
                    pressure=0
                    volume=0
                na=totalsol.get_number_of_atoms()
                ena=en/na
                energy=en
                individ[0]=totalsol[0:nat]
                bul=totalsol[(nat):len(totalsol)]
                STR+='Number of positions = '+repr(len(bul)+len(individ[0]))+'\n'
                individ[0].set_cell(csize)
                indiv=individ[0]
            else:
                en=indiv.get_potential_energy()
                if Optimizer.fitness_scheme == 'enthalpyfit':
                    pressure=indiv.get_isotropic_pressure(indiv.get_stress())
                    cell_max=numpy.maximum.reduce(indiv.get_positions())
                    cell_min=numpy.minimum.reduce(indiv.get_positions())
                    cell=cell_max-cell_min
                    volume=cell[0]*cell[1]*cell[2]
                else: 
                    pressure=0
                    volume=0
                na=indiv.get_number_of_atoms()
                ena=en/na
                energy=ena
                individ[0]=indiv
                bul=0
        else:
            if Optimizer.structure=='Defect' or Optimizer.structure=='Surface':
                if Optimizer.calc_method=='VASP':
                    en=totalsol.get_potential_energy()
                    calcb=Vasp(restart=True)
                    totalsol=calcb.get_atoms()
                    stress=calcb.read_stress()
                else:
                    try:
                        totcop=totalsol.copy()
                        if debug: write_xyz(Optimizer.debugfile,totcop,'Individual sent to lammps')
                        OUT=totalsol.calc.calculate(totalsol)
                        totalsol=OUT['atoms']
                        totalsol.set_pbc(True)
                        if Optimizer.fixed_region:
                            if debug:
                                print 'Energy of fixed region calc = ', OUT['thermo'][-1]['pe']
                            totalsol.set_calculator(Optimizer.static_calc)
                            OUT=totalsol.calc.calculate(totalsol)
                            totalsol=OUT['atoms']
                            totalsol.set_pbc(True)
                            if debug:
                                print 'Energy of static calc = ', OUT['thermo'][-1]['pe']
                        en=OUT['thermo'][-1]['pe']
                        stress=numpy.array([OUT['thermo'][-1][i] for i in ('pxx','pyy','pzz','pyz','pxz','pxy')])*(-1e-4*GPa)
                        #force=numpy.maximum.reduce(abs(totalsol.get_forces()))
                        if debug:
                            write_xyz(Optimizer.debugfile,totalsol,'After Lammps Minimization')
                            Optimizer.debugfile.flush()
                    except Exception, e:
                        os.chdir(cwd)
                        STR+='WARNING: Exception during energy eval:\n'+repr(e)+'\n'
                        f=open('problem-structures.xyz','a')
                        write_xyz(f,totcop,data='Starting structure hindex='+individ.history_index)
                        write_xyz(f,totalsol,data='Lammps Min structure')
                        en=10
                        stress=0
                        f.close()
                if Optimizer.fitness_scheme == 'enthalpyfit':
                    pressure=totalsol.get_isotropic_pressure(stress)
                    cell_max=numpy.maximum.reduce(totalsol.get_positions())
                    cell_min=numpy.minimum.reduce(totalsol.get_positions())
                    cell=cell_max-cell_min
                    volume=cell[0]*cell[1]*cell[2]
                else:
                    pressure=totalsol.get_isotropic_pressure(stress)
                    volume=0
                na=totalsol.get_number_of_atoms()
                ena=en/na
                energy=en
                if Optimizer.structure=='Defect':
                    if Optimizer.fixed_region==True or Optimizer.finddefects==False:
                        individ[0]=totalsol[0:nat]
                        bul=totalsol[(nat):len(totalsol)]
                        individ[0].set_cell(csize)
                    else:
                        if 'FI' in Optimizer.debug:
                            outt=find_defects(totalsol,Optimizer.solidbulk,Optimizer.sf,atomlistcheck=Optimizer.atomlist,trackvacs=Optimizer.trackvacs,trackswaps=Optimizer.trackswaps,debug=Optimizer.debugfile)
                        else:
                            outt=find_defects(totalsol,Optimizer.solidbulk,Optimizer.sf,atomlistcheck=Optimizer.atomlist,trackvacs=Optimizer.trackvacs,trackswaps=Optimizer.trackswaps,debug=False)
                        individ[0]=outt[0]
                        bul=outt[1]
                        individ.vacancies = outt[2]
                        individ.swaps = outt[3]
                        STR += outt[4]
                    indiv=individ[0]
                else:
                    top,bul=find_top_layer(totalsol,Optimizer.surftopthick)
                    indiv=top.copy()
                    individ[0]=top.copy()
            else:
                if Optimizer.calc_method=='VASP':
                    en=totalsol.get_potential_energy()
                    calcb=Vasp(restart=True)
                    totalsol=calcb.get_atoms()
                    stress=calcb.read_stress()
                else:
                    OUT=indiv.calc.calculate(indiv)
                    en=OUT['thermo'][-1]['pe']
                    #indiv.set_positions(OUT['atoms'].get_positions())
                    #indiv.set_cell(OUT['atoms'].get_cell())
                    indiv=OUT['atoms']
                    indiv.set_pbc(True)
                    stress=numpy.array([OUT['thermo'][-1][i] for i in ('pxx','pyy','pzz','pyz','pxz','pxy')])*(-1e-4*GPa)
                if Optimizer.fitness_scheme == 'enthalpyfit':
                    pressure=indiv.get_isotropic_pressure(stress)
                    cell_max=numpy.maximum.reduce(indiv.get_positions())
                    cell_min=numpy.minimum.reduce(indiv.get_positions())
                    cell=cell_max-cell_min
                    volume=cell[0]*cell[1]*cell[2]
                else: 
                    pressure=indiv.get_isotropic_pressure(stress)
                    volume=0
                na=indiv.get_number_of_atoms()
                ena=en/na
                energy=en
                individ[0]=indiv
                bul=0
        STR+='EnergypAtm = '+repr(ena)+'\n'
        if Optimizer.structure=='Crystal':
            STR+='Cell structure = '+repr(check_cell_type(indiv))+'\n'
    
        # Add concentration energy dependence
        if Optimizer.forcing=='Energy_bias':
            n=[0]*len(Optimizer.atomlist)
            for i in range(len(Optimizer.atomlist)):
                n[i]=len([inds for inds in indiv if inds.symbol==Optimizer.atomlist[i][0]])
                n[i]=abs(n[i]-Optimizer.atomlist[i][1])
            factor=sum(n)**3
            energy=(en+factor)/na
        elif Optimizer.forcing=='Chem_pot':
            n=[0]*len(Optimizer.atomlist)
            for i in range(len(Optimizer.atomlist)):
                if Optimizer.structure=='Defect':
                    n[i]=len([inds for inds in totalsol if inds.symbol==Optimizer.atomlist[i][0]])
                else:
                    n[i]=len([inds for inds in indiv if inds.symbol==Optimizer.atomlist[i][0]])
                n[i]=n[i]*Optimizer.atomlist[i][3]
            factor=sum(n)
            energy=(en+factor)/na
            STR+='Energy with Chemical Potential = '+repr(energy[0])+'\n'

        # Add explosion prevention protection
    # 	if 'prevent_explosions' in globals():
    # 		dist=[10]*len(indiv)
    # 		for i in range(len(indiv)):
    # 			for j in range(len(indiv)):
    # 				if i != j:
    # 					dist[j]=indiv.get_distance(i,j,mic=True)
    # 			if min(dist) > 3.5:
    # 				energy+=10
    
        #if Optimizer.structure=='Defect':
        #	individ.force=force
        individ.energy=energy
        individ.buli=bul
        individ.pressure=pressure
        individ.volume=volume
    
        if Optimizer.structure=='Cluster':
            indiv.translate([-250,-250,-250])
        if Optimizer.fingerprinting:
            individ.fingerprint=get_fingerprint(Optimizer,individ,Optimizer.fpbin,Optimizer.fpcutoff)
        if Optimizer.parallel:
            calc.clean()
            signal = 'Evaluated individual '+repr(individ.index)+' on '+repr(rank)+'\n'
            signal +=STR
        else:
            signal=STR

    return energy, bul, individ, signal




'''General switch functions for use in optimizer'''
from crossover_switch import *
from moves_switch import *
#from predator_switch import *
from selection_switch import *
#from lambdacommamu import *
from fitness_switch import *


import random

def fussr(pop,nkeep,Optimizer):
	"""Selection function to employ basic Fixed Uniform Selection Scheme to select and 
	order individuals in a population.
	Scales fuss values
	Picks one fuss point for the selection process
	No constraint on fuss -> population will diverge
	"""
	newpop = []
	#Collect fitnesses
	fits = [ind.fitness for ind in pop]
	#Identify the minimum and maximum fitness and index
	minindex = min(xrange(len(fits)), key=fits.__getitem__)
	minf = min(fits)
	maxf = max(fits)
	#Scale fitnesses to range 0-1
	fits = [(f-minf)/(maxf-minf) for f in fits]
	#Pick FUSS point and calculate distances for each fitness
	pt = random.random()
	distances = [[abs(pt-fits[i]),i] for i in range(len(fits))]
	distances.sort()
	indices = []
	for i in range(nkeep):
		newpop.append(pop[distances[i][1]])
		indices.append(distances[i][1])
	if minindex not in indices:
		rm = random.choice(indices)
		newpop = [pop[inx] for inx in indices if inx != rm]
		newpop.append(pop[minindex])
	return newpop